/*
This file is part of Magstripper, Copyright 2006, 2007.

Magstripper is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Magstripper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package magstripper;
import java.io.Serializable;
import java.util.ArrayList;

public class Cards implements Serializable{
	static final long serialVersionUID = 4362480612757L; //to make extending Serializable happy
	//id, enabled, data, direction, timeallowed, timedenied
	private ArrayList<Object[]> cards = new ArrayList<Object[]>();
	private int currID = 1;
	
	/**
	 * The empty constructor
	 */
	public Cards(){
	}
	
	/**
	 * Adds a card given the object data
	 * @param arg the array of card details
	 */
	public void addCard(Object[] arg){
		cards.add(arg);
	}
	
	/**
	 * Delete a card
	 * @param row the row to delete
	 */
	public void delCard(int row){
		cards.remove(row);
	}
	
	public void delAllCards(){
		cards = new ArrayList<Object[]>();
		currID=1;
	}
	
	/**
	 * Return the next unique ID of cards for this user
	 * @return the ID
	 */
	public int getNextID(){
		return currID++;
	}
	/**
	 * Sets the currID value, used when duplicating users
	 * @param new currID
	 */
	public void setNextID(int i){
		currID = i;
	}
	
	/**
	 * Return all the table data
	 * @return all cards
	 */
	public ArrayList<Object[]> getTableData(){
		return cards;
	}
	
	/**
	 * Increment the use of a specific card, based on it's row
	 * @param row the row to increment
	 */
	public void incrementUse(int row){
		Object[] tmp = cards.get(row);
		tmp[6] = new Integer( ((Integer)tmp[6]).intValue()+1 );
		cards.remove(row);
		cards.add(row, tmp);
	}
	
	/**
	 * Returns the total number of cards attached to this user
	 * @return number of cards
	 */
	public int getNumCards(){
		return cards.size();
	}
	
	/**
	 * Returns the row of data for a specified card
	 * @param i
	 * @return
	 */
	public Object[] getCard(int i){
		return cards.get(i);
	}
	
}