/*
This file is part of Magstripper, Copyright 2006, 2007.

Magstripper is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Magstripper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package magstripper;
import java.util.Observable;
import java.util.Vector;
//filter levels
//0 - Error
//1 - Warning
//2 - Info
//3 - Debug

public class ErrorLog extends Observable{
	private Vector<String> log; //the object of error messages, one string per entry
	private int debugnum = 0, infonum = 0, warnnum = 0, errornum = 0; //the number or each level of filter messages
	
	/**
	 * constructor for the error log, initializes the object that holds the error messages.
	 */
	public ErrorLog(){
		log = new Vector<String>();
	}
	
	//returns true if added
	/**
	 * Appends a status message to the errorlog object
	 * @return true if the messages was added successfully, false otherwise
	 */
	public boolean addMsg(String s){
		boolean retval = false;
		//synchronized not needed
		if(s.startsWith("Error:")){ errornum++;retval=true;}
		else if(s.startsWith("Warning:")){ warnnum++;retval=true;}
		else if(s.startsWith("Info:")){ infonum++;retval=true;}
		else if(s.startsWith("Debug:")){ debugnum++;retval=true;}

		if(retval){log.add(s);setChanged();notifyObservers("log");}
		else{ addMsg("Warning: New Log Entry Lacks Proper Prefix; "+s);}
		return retval;
	}
	
	/**
	 * Returns entries at or lower (more severe) than the current level.
	 * @param level 0=Error 1=Warning 2=Info 3=Debug
	 * @return string of requsted entries, newline at the end of each entry
	 */
	public String[] getEntries(int level){//
		int numret = ((level>=3)?debugnum:0) + ((level>=2)?infonum:0) + ((level>=1)?warnnum:0) + ((level>=0)?errornum:0);
		Object[] full = log.toArray();
		String[] retval = new String[ numret ];
		numret=0; //use as a position marker for returned array
		
		for(int i=0;i<full.length;i++){
			String line = (String)full[i];
			if(line.startsWith("Error:") && level >= 0){ retval[numret] = line+'\n'; numret++; }
			else if(line.startsWith("Warning:") && level >= 1){ retval[numret] = line+'\n'; numret++; }
			else if(line.startsWith("Info:") && level >= 2){ retval[numret] = line+'\n'; numret++; }
			else if(line.startsWith("Debug:") && level >= 3){ retval[numret] = line+'\n'; numret++; }	
		}
		
		return retval;
	}
	
	public String[] getEntries(){
		return getEntries(3);
	}
	
	/**
	 * Returns the number of entries at the requested log level and below (more severe)
	 * @param level 0=Error 1=Warning 2=Info 3=Debug
	 * @return the number of entries
	 */
	public int getNumEntries(int level){
		return ((level>=3)?debugnum:0) + ((level>=2)?infonum:0) + ((level>=1)?warnnum:0) + ((level>=0)?errornum:0);
	}
	
	/**
	 * Returns the number of debug entries
	 * @return number of debug entries
	 */
	public int getDebugNum(){ return debugnum; }
	/**
	 * Returns the number of info entries
	 * @return number of info entries
	 */
	public int getInfoNum(){ return infonum; }
	/**
	 * Returns the number of warning entries
	 * @return number of warning entries
	 */
	public int getWarningNum(){ return warnnum; }
	/**
	 * Returns the number of error entries
	 * @return number of error entries
	 */
	public int getErrorNum(){ return errornum; }
	/**
	 * Returns the number of entries
	 * @return number of entries
	 */
	public int getNumEntries(){ return log.size(); }
	
}