/*
This file is part of Magstripper, Copyright 2006, 2007.

Magstripper is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Magstripper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package magstripper;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultFocusManager;
import javax.swing.ImageIcon;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
//the controller
public class Magstripper {
	private static MagstripperModel model;
	private static MagstripperGUI view;
	private static Cli cli;
	private static boolean climode = false;
	/**
	 * main method for magstripper, launches controller
	 * @param args[] arguements are not parsed
	 */
	public static void main(String args[]){
		if(args.length > 0){
			//command line mode
			if( args.length == 1 && ( args[0].equals("-c") || args[0].equals("--c") || args[0].equals("-cli") || args[0].equals("--cli") || 
					args[0].equals("-command") || args[0].equals("--command") || args[0].equals("-commandline") || args[0].equals("--commandline") ) ){
				climode = true;
				
			}
		}
		new Magstripper(climode); //create the controller in a non static way
		
	}
	
	/**
	 * The constructor for the controller. Creates the model and view, and attaches listeners to the view, then displays the GUI.
	 */
	public Magstripper(boolean climode) {
		if(climode) {
			model = new MagstripperModel();
			cli = new Cli(model);
			micListener ml = new micListener();
			new openListener();
			ml.startListening();
			
		}else{
			Loading loadwin = new Loading();
			model = new MagstripperModel();
			JFrame.setDefaultLookAndFeelDecorated(true); //must be called before the jframe is created, in this case by super() in the contructor
			view = new MagstripperGUI(model);
			view.createGUI();
			view.addWindowListener(new exitListener());
			view.addExitListener(new exitListener());
			loadwin.setPercent(15);
			view.addOpenListener(new openListener()); //long pause
			loadwin.setPercent(30);
			view.addSystemLogComboBoxListener(new systemLogComboBoxListener()); //system log should be the first listener incase others cause errors
			view.addConsoleLogComboBoxListener(new consoleLogComboBoxListener());
			view.addMicListener(new micListener());
			view.addUserTableListener(new userTableListener());
			view.addLockListener(new lockListener());
			view.addConversionToolListener(new conversionToolListener());
			view.addCharacterTableListener(new characterTableListener());
			view.addAboutListener(new aboutListener());
			view.addSaveListener(new saveListener()); //long pause
			view.addUserAddListener(new userAddListener());
			view.addUserDelListener(new userDelListener());
			loadwin.setPercent(45);
			view.addLockScreenListener(new lockScreenListener());
			view.addUserOpenListener(new userOpenListener()); //long pause
			loadwin.setPercent(60);
			view.addUserSaveListener(new userSaveListener()); //long pause
			view.addUserClearListener(new userClearListener());
			view.addPrefListener(new prefListener());
			view.addCardAddListener(new cardAddListener());
			view.addCardDelListener(new cardDelListener());
			view.addSwipeListSelectionListener(new swipeListSelectionListener());
			view.addCharListSelectionListener(new charListSelectionListener());
			loadwin.setPercent(75);
			view.addSaveConsoleLogListener(new saveConsoleLogListener()); //long pause
			view.addUserTableTooltipListener(new userTableTooltipListener());
			view.addUserTablePopupListener(new userTablePopupListener());
			view.addSaveSystemLogListener(new saveSystemLogListener()); //long pause
			loadwin.setPercent(90);
			view.addTableTabListener(new tableTabListener()); //must be after all listeners that switch tabs
			view.showGUI();
			loadwin.doneLoading();
		}
	}
	
	/**
	 * Listener class that is notified when the selection of the usertable is changed. 
	 * When this happens the card table is changed to display cards by this user
	 */
	private class userTableListener implements ListSelectionListener{ //listener for selection updates to user table
		
		/**
		 * Method called when the selection of the user table is changed
		 */
		public void valueChanged(ListSelectionEvent e) { //when the selection changes
			int sel = ((ListSelectionModel)e.getSource()).getMinSelectionIndex(); //which row is selected
			if(!e.getValueIsAdjusting() && sel<model.getTableModel().getRowCount() && sel > -1){//make sure the selection is done, double check we selected a valid row, make sure we selected a row
				Object cell = model.getTableModel().getValueAt(sel, UserTableModel.ID); //get the ID of the selected row
				if(cell != null && cell.getClass() == Integer.class){ //if the row is selected and has a user in it
					model.getCardTableModel().updateTable( ((Integer)cell).intValue() ); //update the card table based on this user
				}else{//clear the card table
					model.getCardTableModel().updateTable(-1);
					model.getLog().addMsg("Debug: Clearing Card Table Display");
				}
			}
		}
	}
	
	/**
	 * When the button is clicked to add a user this listener is called. It displays a dialog for additional information, parses the fields, and adds a user to the table.
	 */
	private class cardAddListener extends KeyAdapter implements ActionListener{
		JDialog jd;
		JTextField dataf;
		JComboBox dir, start1, start2, end1, end2;
		JButton addb;
		
		public void keyPressed(KeyEvent e){
			DefaultFocusManager focusManager = new DefaultFocusManager();
			if(e.getKeyCode() == KeyEvent.VK_DOWN)
				focusManager.focusNextComponent();
			else if(e.getKeyCode() == KeyEvent.VK_UP)
				focusManager.focusPreviousComponent();
			else if(e.getKeyCode() == KeyEvent.VK_ENTER)
				addb.doClick();
			else if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
				jd.dispose();
		}
		
		/**
		 * Method called when the add user button is pressed
		 */
		public void actionPerformed(ActionEvent e){ //when the button was clicked
			if( view.getUserTableRowSelected() > -1 && model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID) != null &&
					model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID).getClass() == Integer.class ){ //if something's selected and it's got a valid id
				JDialog.setDefaultLookAndFeelDecorated(true);
				jd = new JDialog(view, "Add Card", true); //create a dialog to popup
				JPanel jdPanel = new JPanel(new BorderLayout()); //the main panel
				JPanel inputPanel = new JPanel(new GridBagLayout()); //the center panel with all the input fields
				JLabel data = new JLabel(" Data  ");
				JLabel direction = new JLabel(" Direction  ");
				JLabel stime = new JLabel(" Start Time  ");
				JLabel etime = new JLabel(" End Time  ");
				dataf = new JTextField(12);
				dataf.addKeyListener(this);
				if(view.getSwipeSelected() != null) //if there is a swipe selected, fill in the default "Data" box with this swipe information
					dataf.setText(model.getSwipe(view.getSwipeSelected()).getDecodedString()); //set the data text box to the last ascii swipe
				String[] dirs = {"Both", "Forward", "Backward"};
				dir = new JComboBox(dirs);
				dir.addKeyListener(this);
				String[] hours = {"00:","01:","02:","03:","04:","05:","06:","07:","08:","09:","10:","11:","12:","13:","14:","15:","16:","17:","18:","19:","20:","21:","22:","23:"};
				start1 = new JComboBox(hours);start2 = new JComboBox(hours);
				start1.addKeyListener(this);start2.addKeyListener(this);
				String[] mins = {"00","05","10","15","20","25","30","35","40","45","50","55","60"};
				end1 = new JComboBox(mins);end2 = new JComboBox(mins);
				end1.addKeyListener(this);end2.addKeyListener(this);
				GridBagConstraints gbc = new GridBagConstraints();gbc.fill = GridBagConstraints.HORIZONTAL;
				gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 1;inputPanel.add(data,gbc); //add all the input elements to the panel using a gridbag layout
				gbc.gridx = 1;gbc.gridy = 0;gbc.gridwidth = 2;inputPanel.add(dataf,gbc);
				gbc.gridx = 0;gbc.gridy = 1;gbc.gridwidth = 1;inputPanel.add(direction,gbc);
				gbc.gridx = 1;gbc.gridy = 1;gbc.gridwidth = 2;inputPanel.add(dir,gbc);
				gbc.gridx = 0;gbc.gridy = 2;gbc.gridwidth = 1;inputPanel.add(stime,gbc);
				gbc.gridx = 1;gbc.gridy = 2;gbc.gridwidth = 1;inputPanel.add(start1,gbc);
				gbc.gridx = 2;gbc.gridy = 2;gbc.gridwidth = 1;inputPanel.add(end1,gbc);
				gbc.gridx = 0;gbc.gridy = 3;gbc.gridwidth = 1;inputPanel.add(etime,gbc);
				gbc.gridx = 1;gbc.gridy = 3;gbc.gridwidth = 1;inputPanel.add(start2,gbc);
				gbc.gridx = 2;gbc.gridy = 3;gbc.gridwidth = 1;inputPanel.add(end2,gbc);
				gbc.gridx = 0;gbc.gridy = 4;gbc.gridwidth = 1;inputPanel.add(new JLabel(" "),gbc);
				addb = new JButton("Add Card"); //buttons on the bottom
				addb.addKeyListener(this);
				JButton cancelb = new JButton("Cancel");
				cancelb.addKeyListener(this);
				JPanel buttonPanel = new JPanel();
				buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
				buttonPanel.add(Box.createHorizontalGlue());
				buttonPanel.add(addb);buttonPanel.add(Box.createRigidArea(new Dimension(5,5)));buttonPanel.add(cancelb);
				addb.addActionListener(new addCardListenerDialog()); //add listeners for the two buttons in this dialog box
				cancelb.addActionListener(new cancelUserListenerDialog());
				jdPanel.add(inputPanel,BorderLayout.CENTER); //add the elements to the main pane and display it
				jdPanel.add(buttonPanel, BorderLayout.SOUTH);
				jd.setContentPane(jdPanel);	
				jd.pack();
				jd.setResizable(false);
				Dimension d = view.getSize();
				Point p = view.getLocationOnScreen();
				jd.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
				jd.setVisible(true);
			}else{ //you have to select a user to add a card
				JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane.showMessageDialog(view, "Please Select A Single User To Add A Card", "Card Add Error", JOptionPane.WARNING_MESSAGE);
				model.getLog().addMsg("Debug: Could Not Add A Card, First Select A User");
			}
		}
		/**
		 * Internal listener class used for the add card button in this dialog
		 */
		private class addCardListenerDialog implements ActionListener{
			
			/**
			 * Method called when the add card button is pressed
			 */
			public void actionPerformed(ActionEvent e){
				CardTableModel ctm = model.getCardTableModel();
				Integer curUser = (Integer)model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID);
				int h = Integer.valueOf( ((String)start1.getSelectedItem()).substring(0, 2)).intValue();//strip the : and convert it to an int
				int m = Integer.valueOf( ((String)end1.getSelectedItem())).intValue();
				int h1 = Integer.valueOf( ((String)start2.getSelectedItem()).substring(0, 2)).intValue();//strip the : and convert it to an int
				int m1 = Integer.valueOf( ((String)end2.getSelectedItem())).intValue();
				Time stime = new Time(h,m);
				Time etime = new Time(h1,m1);
				if(stime.compareTo(etime) > 0){//end time before starttime
					JDialog.setDefaultLookAndFeelDecorated(true);
					JOptionPane.showMessageDialog(jd, "The Start Time Must Be The Same, Or Before The End Time", "Card Add Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Add Card, Start Time Must Be The Same, Or Before The End Time");
					return;
				}else if(dataf.getText().equals("")){ //data cant be empty
					JDialog.setDefaultLookAndFeelDecorated(true);
					JOptionPane.showMessageDialog(jd, "The Data Field Must Contain A String", "Card Add Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Debug: Could Not Add Card, The Data Field Must Contain A String");
					return;
				}
				String tmp = dataf.getText();
				boolean outofrange = false;
				for(int i=0;i<tmp.length();i++){
					int asciival = tmp.charAt(i);
					if(asciival > 95 || asciival < 32){//char out of range of standard charset
						outofrange = true;
					}
				}
				if(outofrange){//if there's a char out of range, warn
					Object[] opt = {"Continue", "Cancel"};
					if(1 == JOptionPane.showOptionDialog(view, " Data Contains A Character Outside\n Of The Standard Characterset\n(Use Capital Characters)", "Data Out Of Range", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
						return;
				}
				MessageDigest digest = null;
				try {
					digest = MessageDigest.getInstance("MD5");
				} catch (NoSuchAlgorithmException e1) {
					model.getLog().addMsg("Error: Could Not Load MD5 Hashing Function");jd.dispose();
					JOptionPane.showMessageDialog(view, "Can Not Load The MD5 Hashing Function", "Hash Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				int salt = (int)(Math.random()*(model.getSaltRange()+1));
				byte[] suffix = tmp.getBytes();
				byte[] plain = new byte[suffix.length+2];//prepend 2 bytes of salt
				System.arraycopy(suffix, 0, plain, 2, suffix.length);
				plain[0] = (byte)((salt & 0xff00)>>>8);//high order byte of salt, shift down to a byte and add it in the first position
				plain[1] = (byte)(salt & 0xff);//low order byte of salt, cut out 
				byte[] hashed = digest.digest(plain);//hash it
				
				String hashedString = "";
				for(int i=0;i<16;i++){
					int high = (hashed[i] & 0xf0)>>>4;//split up the current byte
					int low = (hashed[i]) & 0x0f;
					hashedString += Integer.toString(high, 16);//append the nibble to the string in hex
					hashedString += Integer.toString(low, 16);
				}
				if(!model.getSecLog())
					model.getLog().addMsg("Debug: The Hash Of "+tmp+" = "+hashedString);
				Integer id = new Integer(ctm.getCards(curUser).getNextID());
				Object[] addme = { id, new Boolean(false), hashedString, dir.getSelectedItem(), 
						stime, etime, new Integer(0) };
				ctm.addRow(addme, curUser.intValue() );
				model.getTableModel().incrementCards(view.getUserTableRowSelected());
				view.updateCardCount();
				model.getLog().addMsg("Info: Adding Card: "+id.intValue()+" For User: "+curUser.intValue());
				jd.dispose();
			}
		}
		/**
		 * Internal listener class used for the cancel button in this dialog
		 */
		private class cancelUserListenerDialog implements ActionListener{
			
			/**
			 * Method called when the cancel button is pressed 
			 */
			public void actionPerformed(ActionEvent e){
				jd.dispose();
			}
		}
	}
	
	/**
	 * Listener called when the card delete button is pressed
	 */
	private class cardDelListener implements ActionListener{
		
		/**
		 * Method called when the card delete button is pressed
		 */
		public void actionPerformed(ActionEvent e){
			if(view.getCardTableRowSelected() == -1){ //first select a card you want to delete
				JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane.showMessageDialog(view, "Please Select A Card For Deletion", "Card Delete Error", JOptionPane.WARNING_MESSAGE);
				model.getLog().addMsg("Debug: Could Not Delete Card, Must Select A Card");
			}else{ //delete that card
				model.getLog().addMsg("Info: Deleting Card At Row: "+view.getCardTableRowSelected()+" For User: "+((Integer)model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID)).intValue());
				model.getCardTableModel().delRow(view.getCardTableRowSelected(), ((Integer)model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID)).intValue() );
				model.getTableModel().decrementCards(view.getUserTableRowSelected());
				view.updateCardCount();
			}
		}
	}
	
	/**
	 * Listener called with the add user button is pressed. Creates a dialog and prompts for more info.
	 */
	private class userAddListener extends KeyAdapter implements ActionListener {
		JTextField firstText, lastText, timeText;
		JDialog jd;
		JButton addb;
		
		/**
		 * Listener called when a key is pressed, used for enter/esc/up/down shortcuts from any input box
		 */
		public void keyPressed(KeyEvent e){
			DefaultFocusManager focusManager = new DefaultFocusManager();
			if(e.getKeyCode() == KeyEvent.VK_DOWN)
				focusManager.focusNextComponent();
			else if(e.getKeyCode() == KeyEvent.VK_UP)
				focusManager.focusPreviousComponent();
			else if(e.getKeyCode() == KeyEvent.VK_ENTER)
				addb.doClick();
			else if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
				jd.dispose();
		}
		
		/**
		 * Method called with the add user button is pressed. Creates a dialog and prompts for more info.
		 */
		public void actionPerformed(ActionEvent e){
			JDialog.setDefaultLookAndFeelDecorated(true);
			jd = new JDialog(view, "Add User", true); //create the dialog
			JPanel jdPanel = new JPanel(new BorderLayout());
			JPanel inputPanel = new JPanel(new GridBagLayout());
			JLabel first = new JLabel(" First Name  "); //create the labels
			JLabel last = new JLabel(" Last Name  ");
			JLabel time = new JLabel(" Unlock Time  ");
			firstText = new JTextField(12); //create the text fields
			firstText.addKeyListener(this);
			lastText = new JTextField(12);
			lastText.addKeyListener(this);
			timeText = new JTextField(12);
			timeText.addKeyListener(this);
			timeText.setText(model.getLockTime()+""); //fill in the default value for unlock time
			GridBagConstraints gbc = new GridBagConstraints();gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 1;inputPanel.add(first,gbc); //add all the elements using a gridbag
			gbc.gridx = 1;gbc.gridy = 0;gbc.gridwidth = 2;inputPanel.add(firstText,gbc);
			gbc.gridx = 0;gbc.gridy = 1;gbc.gridwidth = 1;inputPanel.add(last,gbc);
			gbc.gridx = 1;gbc.gridy = 1;gbc.gridwidth = 2;inputPanel.add(lastText,gbc);
			gbc.gridx = 0;gbc.gridy = 2;gbc.gridwidth = 1;inputPanel.add(time,gbc);
			gbc.gridx = 1;gbc.gridy = 2;gbc.gridwidth = 2;inputPanel.add(timeText,gbc);
			gbc.gridx = 0;gbc.gridy = 3;gbc.gridwidth = 1;inputPanel.add(new JLabel(" "),gbc);
			addb = new JButton("Add User"); //create the two buttons
			JButton cancelb = new JButton("Cancel");
			addb.addKeyListener(this);
			cancelb.addKeyListener(this);
			JPanel buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			buttonPanel.add(Box.createHorizontalGlue());
			buttonPanel.add(addb);buttonPanel.add(Box.createRigidArea(new Dimension(5,5)));buttonPanel.add(cancelb);
			addb.addActionListener(new addUserListenerDialog()); //add the listeners to the buttons
			cancelb.addActionListener(new cancelUserListenerDialog());
			jdPanel.add(inputPanel,BorderLayout.CENTER); //pack and display it
			jdPanel.add(buttonPanel, BorderLayout.SOUTH);
			jd.setContentPane(jdPanel);	
			jd.pack();
			jd.setResizable(false);
			Dimension d = view.getSize();
			Point p = view.getLocationOnScreen();
			jd.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
			jd.setVisible(true);
		}
		
		/**
		 * Listener for the add user button in this add user dialog
		 */
		private class addUserListenerDialog implements ActionListener{ //adduser button displayed in dialog for adduser
			
			/**
			 * Method when the add user button is pressed in the add user dialog
			 */
			public void actionPerformed(ActionEvent e){
				UserTableModel tm = model.getTableModel();
				//check the input
				try{
					new Integer(timeText.getText());
				}catch(NumberFormatException nfe){
					JOptionPane.showMessageDialog(jd, "Unlock Time Must Be Betweeen 100ms And 43200000ms (12hr)", "User Add Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Parse Default Unlock Time, Must Be Between 100ms And 43200000ms (12hr)");
					return;
				}
				if( new Integer(timeText.getText()).intValue() > 43200000 || new Integer(timeText.getText()).intValue() < 100 ){
					JDialog.setDefaultLookAndFeelDecorated(true);
					JOptionPane.showMessageDialog(jd, "Unlock Time Must Be Betweeen 100ms And 43200000ms (12hr)", "User Add Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Add User, Unlock Time not between 100 - 43200000ms");
					return;
				}
				Integer id = new Integer(tm.getNextID());
				Object[] addme = { id, new Boolean(false), firstText.getText(), lastText.getText(), new Integer(0), new Integer(timeText.getText())};
				tm.addRow(addme);
				view.updateUserCount();//update the status bar
				view.setTableSelection(tm.getRowsDefined()-1); //highlight the newly added row
				model.getLog().addMsg("Info: Adding User: "+id.intValue());
				jd.dispose();
			}
		}
		
		/**
		 * Listener for the cancel button
		 */
		private class cancelUserListenerDialog implements ActionListener{//cancel button displayed in dialog for adduser
			
			/**
			 * Method for the cancel button
			 */
			public void actionPerformed(ActionEvent e){
				jd.dispose();
			}
		}
	}
	
	/**
	 * Listener called when the user delete button is pressed.
	 */
	private class userDelListener implements ActionListener{
		
		/**
		 * Method called when the user delete button is pressed.
		 */
		public void actionPerformed(ActionEvent e){
			if(view.getUserTableRowSelected() == -1){
				JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane.showMessageDialog(view, "Please Select A Single User For Deletion", "User Delete Error", JOptionPane.WARNING_MESSAGE);
			}else{ //delete the user
				if(model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID).getClass() == Integer.class){
					model.getLog().addMsg("Info: Deleting User: "+model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID));
				}
				model.getCardTableModel().updateTable(-1); //clear the card table
				model.getTableModel().delRow(view.getUserTableRowSelected());//delete the row
				view.updateCardCount();
				view.updateUserCount();
			}
		}
	}
	
	/**
	 * Listener called when the table load button is pressed. 
	 */
	private class userOpenListener implements ActionListener{ 
		final JFileChooser fc = new JFileChooser();
		/**
		 * Constructor for things only executed once
		 */
		userOpenListener(){
			fc.addChoosableFileFilter(new MagstripperFilter());
		}
		/**
		 * Method called when the table load button is pressed
		 */
		public void actionPerformed(ActionEvent e){
			if(model.getTableModel().getRowsDefined() > 0){ //if there's user information in the table
				JDialog.setDefaultLookAndFeelDecorated(true);
				Object[] opt = {"Discard It", "Cancel"};
				if(1 == JOptionPane.showOptionDialog(view, " You have User Information Open\n Are You Sure You Want Overwrite It?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
					return;
			}
			
			JDialog.setDefaultLookAndFeelDecorated(true);
			fc.setFileHidingEnabled(true);
			
			int clicked = fc.showOpenDialog(view);
			if(clicked == JFileChooser.APPROVE_OPTION){ //which button was clicked
				File f = fc.getSelectedFile();
				model.getLog().addMsg("Info: Opening Table Data From "+f.getName());
				try {
					ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
					UserTableModel utm = ((UserTableModel)ois.readObject());
					CardTableModel ctm = ((CardTableModel)ois.readObject());
					ois.close();
					model.setUserTableModel(utm);//update the models view of the table
					model.setCardTableModel(ctm);
					view.unclearTableEditing(); //update the table (swing component)
					view.updateCardTable();
					model.getCardTableModel().updateTable(-1);//clear the card table since the user table wont be selected and could cause problems
					utm.setMagstripperModel(model); //update the tables view of the model
					view.updateUserCount();
					view.updateCardCount();
				} catch (FileNotFoundException e1) {
					model.getLog().addMsg("Error: FileNotFoundException thrown when trying to open "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Open The Specified File\nDouble Check Your Permissions", "File Open Error",JOptionPane.ERROR_MESSAGE);
					return;
				} catch (IOException e1) {
					model.getLog().addMsg("Error: IOException thrown when trying to open "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Open The Specified File\nI/O Problem Encountered", "File Open Error",JOptionPane.ERROR_MESSAGE);
					return;
				} catch (ClassNotFoundException e1){
					model.getLog().addMsg("Error: ClassNotFoundException thrown when trying to open "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Open The Specified File\nFiles Must Be Saved/Opened In The Same Version", "File Open Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
			}//end approve option clicked
		}
	}
	
	/**
	 * Listener called when the table save button is pressed.
	 */
	private class userSaveListener implements ActionListener{
		final JFileChooser fc = new JFileChooser();
		File def = new File(".mag");
		/**
		 * Constructor for things only executed once
		 */
		userSaveListener(){
			fc.addChoosableFileFilter(new MagstripperFilter());
		}
		
		/**
		 * Method called when the table save button is pressed
		 */
		public void actionPerformed(ActionEvent e){
			JDialog.setDefaultLookAndFeelDecorated(true);
			if(model.getTableModel().getRowsDefined() < 1 ){
				JOptionPane.showMessageDialog(view, "User Table Must Contain An Entry Before It Can Be Saved", "Table Save Error", JOptionPane.WARNING_MESSAGE );
				return;
			}
			
			fc.setSelectedFile(def);
			int clicked = fc.showSaveDialog(view);
			if(clicked == JFileChooser.APPROVE_OPTION){
				File f = fc.getSelectedFile();
				if(f.exists()){ //file exists will overwrite with intervention
					JDialog.setDefaultLookAndFeelDecorated(true);
					Object[] opt = {"Overwrite It", "Cancel"};
					if(1 == JOptionPane.showOptionDialog(view, " File Exists, Overwrite It?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
						actionPerformed(null);
				}
				model.getLog().addMsg("Info: Saving Table Data To "+f.getName());
				try {
					ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(f));
					view.clearTableEditing();//clear the table model because celleditorremover cant be serialized
					oos.writeObject(model.getTableModel());
					oos.writeObject(model.getCardTableModel());
					oos.close();
					view.unclearTableEditing();//replace the editor
					
				} catch (FileNotFoundException e1) {
					model.getLog().addMsg("Warning: FileNotFoundException Caught When Saving File "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Save The Specified File\nDouble Check Your Permissions", "File Save Error",JOptionPane.ERROR_MESSAGE);
					return;
				} catch (IOException e1) {
					model.getLog().addMsg("Warning: IOException Caught When Saving File "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Save The Specified File\nI/O Problem Encountered", "File Save Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
		}
	}
	
	/**
	 * Listener called when the clear table button is pressed
	 */
	private class userClearListener implements ActionListener{
		/**
		 * Called when the button is pressed
		 */
		public void actionPerformed(ActionEvent a) {
			JDialog.setDefaultLookAndFeelDecorated(true);
			if(model.getTableModel().getRowsDefined() < 1 ){
				JOptionPane.showMessageDialog(view, "User Table Must Contain An Entry Before It Can Be Cleared", "Table Clear Error", JOptionPane.WARNING_MESSAGE );
				return;
			}
			Object[] opt = {"Delete Table", "Cancel"};
			if(1 == JOptionPane.showOptionDialog(view, " Remove All Users And Cards In The Table?\nThis Cannot Be Undone", "Clear Table", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
				return;
			while(model.getTableModel().getRowsDefined() > 0){//loop as long as there's more rows defined
				model.getTableModel().delRow(0);
			}
			view.updateUserCount();//update the status bar
			view.updateCardCount();
			
			model.getCardTableModel().updateTable(-1);//clear the card table
			model.getLog().addMsg("Info: Clearing The User Table And Card Table");
		}
	}
	
	/**
	 * Listener called with the mic status button is clicked, and when the zero level or noise thresholds are changed
	 */
	private class micListener implements ActionListener, Observer{
		MicIn main;
		micListener(){
			main = new MicIn(model);
			main.setPriority(Thread.MAX_PRIORITY);
			main.start();
			model.addObserver(this);
		}
		
		/**
		 * Called when the mic button is clicked
		 * @param actionevent created from the button
		 */
		public void actionPerformed(ActionEvent e){
			
			if(main.getMic() == null){//mic could not be initialized
				JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane.showMessageDialog(view, "Could Not Start Mic Input\nCheck Error Log For Mic Initialization Problems", "Mic Input Error", JOptionPane.ERROR_MESSAGE);
			}else if(model.getListeningState()){//stop listening
				stopListening();
				view.setListening(model.getListeningState());
				model.getLog().addMsg("Info: Suspending Listening From Microphone Input");
			}else{//start listening
				startListening();
				view.setListening(model.getListeningState());
				model.getLog().addMsg("Info: Starting Listening From Microphone Input");
			}
		}
		
		public void startListening(){
			model.setListening(true);
			main.suspendListening(false);
		}
		
		public void stopListening(){
			model.setListening(false);
			main.suspendListening(true);
		}
		
		/**
		 * Listener called with the zero level or noise threshold is 
		 */
		public void update(Observable obs, Object obj){
			if(obj == "zerolevelautomatic" && model.getZeroAutomatic()){ //zero level automatic was changed, only care if it was set back to automatic. otherwise we'll get a message when the static value is set
				main.setZeroLevel(main.recalculateZeroLevel());
				//main.setThresholds();
				//model.getLog().addMsg("Info: Changing Zero Level To: "+model.getZeroLevel()+" Thresholds ");
			}else if(obj == "zerolevel"){
				if(!model.getZeroAutomatic()){ //zerolevel statically set
					main.setZeroLevel((short)model.getZeroLevel());
					//main.setThresholds();
					//model.getLog().addMsg("Info: Changing Zero Level To: "+model.getZeroLevel());
				}
			}else if(obj == "thresholds"){
				main.setThresholds(); //system log is updated in micin.java
			}
		}
	}
	
	//TODO stats update, leading one bit, check crc, generate wav?
	private class conversionToolListener implements ActionListener{
		JDialog jd;
		JLabel tbpc, tstartfound, tendfound, tleadingzeros, ttrailingzeros, tbinarybackwards, tleadingone, tasciilength, tbinarylength, tlrcfound, tlrcpassed, tpeaksrequired;
		JRadioButton bpcauto, bpcfive, bpcseven;
		JTextPane ascii, binary;
		boolean asciiclear = true, binaryclear = true;
		JButton atob, btoa;
		
		conversionToolListener(){
			JDialog.setDefaultLookAndFeelDecorated(true);
			jd = new JDialog(view, "Conversion Tool", true);
			atob = new JButton("ASCII -> Binary ", new ImageIcon(MagstripperGUI.class.getResource("icons/arrow_bottom.png")));
			atob.setToolTipText("Convert ASCII Text To Binary Data");
			btoa = new JButton("Binary -> ASCII ", new ImageIcon(MagstripperGUI.class.getResource("icons/arrow_top.png")));
			btoa.setToolTipText("Convert Binary Data to ASCII Data");
			ascii = new JTextPane();
			binary = new JTextPane();
		}
		
		/**
		 * Listener called when Conversion Tool is selected
		 */
		public void actionPerformed(ActionEvent arg0) {
			//TODO remove this after it's fully tested
			JOptionPane.showMessageDialog(jd, "This is not 100% working in this alpha version\nYou should check the table against any conversions to be certain", "Conversion Info", JOptionPane.INFORMATION_MESSAGE);
			Insets noboarder = new Insets(0,0,0,0);
			btoa.setMargin(noboarder);
			atob.setMargin(noboarder);
			JPanel jdPanel = new JPanel(new BorderLayout());
			JToolBar toolbar = new JToolBar("Conversion Toolbar");
			
			toolbar.add(atob);
			toolbar.addSeparator();
			toolbar.add(btoa);
			conversionActionListener cal = new conversionActionListener();
			atob.addActionListener(cal);
			atob.addKeyListener(cal);
			btoa.addActionListener(cal);
			btoa.addKeyListener(cal);
			jdPanel.add(toolbar, BorderLayout.NORTH);
			
			ascii.addKeyListener(cal);
			binary.addKeyListener(cal);
			ascii.setInputVerifier(new asciiVerifier());
			binary.setInputVerifier(new binaryVerifier());
			JScrollPane asciiscroll = new JScrollPane(ascii, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			JScrollPane binaryscroll = new JScrollPane(binary, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			JLabel asciihead = new JLabel("  ASCII Text  ");
			JLabel binaryhead = new JLabel("  Binary Data  ");
			asciiscroll.setColumnHeaderView(asciihead);
			binaryscroll.setColumnHeaderView(binaryhead);
			ascii.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, jd.getFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS));
	        binary.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, jd.getFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS));
			JSplitPane center = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, asciiscroll, binaryscroll);
			center.setResizeWeight(.2);
			jdPanel.add(center, BorderLayout.CENTER);
			
			JPanel side = new JPanel();
			side.setLayout(new BoxLayout(side, BoxLayout.Y_AXIS));
			
			JPanel options = new JPanel(new GridBagLayout());
			options.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Options"));
			ButtonGroup bgbpc = new ButtonGroup();
			JRadioButton bpcauto = new JRadioButton(" Auto ");
			bpcauto.setMargin(noboarder);
			bpcauto.addKeyListener(cal);
			JRadioButton bpcfive = new JRadioButton(" 5 ");
			bpcfive.setMargin(noboarder);
			bpcfive.addKeyListener(cal);
			JRadioButton bpcseven = new JRadioButton(" 7 ");
			bpcseven.setMargin(noboarder);
			bpcseven.addKeyListener(cal);
			bpcauto.addActionListener(cal);
			bpcfive.addActionListener(cal);
			bpcseven.addActionListener(cal);
			bgbpc.add(bpcauto);bgbpc.add(bpcfive);bgbpc.add(bpcseven);
			bpcauto.setSelected(true);
			JLabel bpcoption = new JLabel(" Bits Per Char: ");
			GridBagConstraints bpcgrid = new GridBagConstraints();
			bpcgrid.fill = GridBagConstraints.HORIZONTAL; bpcgrid.weightx = 1;
			bpcgrid.gridx = 0;bpcgrid.gridy = 0;bpcgrid.gridwidth = 2;options.add(bpcoption,bpcgrid);
			bpcgrid.gridx = 2;bpcgrid.gridy = 0;bpcgrid.gridwidth = 1;options.add(bpcauto,bpcgrid);
			bpcgrid.gridx = 2;bpcgrid.gridy = 1;bpcgrid.gridwidth = 1;options.add(bpcfive,bpcgrid);
			bpcgrid.gridx = 2;bpcgrid.gridy = 2;bpcgrid.gridwidth = 1;options.add(bpcseven,bpcgrid);
			
			JPanel stats = new JPanel(new GridBagLayout());
			stats.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Stats"));
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.fill = GridBagConstraints.HORIZONTAL;gbc.weightx = 1;
			tbpc = new JLabel(" 0 ");tstartfound = new JLabel(" No ");tendfound = new JLabel(" No ");tleadingzeros = new JLabel(" 0 ");
			ttrailingzeros = new JLabel(" 0 ");tbinarybackwards = new JLabel(" No ");tleadingone = new JLabel(" No ");tasciilength = new JLabel(" 0 ");
			tbinarylength = new JLabel(" 0 ");tlrcfound = new JLabel(" No ");tlrcpassed = new JLabel(" No ");tpeaksrequired = new JLabel(" 0 ");
			JLabel lbpc = new JLabel(" Bits Per Char: ");JLabel lstartfound = new JLabel(" Start Sentinel:  ");
			JLabel lendfound = new JLabel(" End Sentinel:  ");JLabel lleadingzeros = new JLabel(" Leading Zeros:  ");
			JLabel ltrailingzeros = new JLabel(" Trailing Zeros:  ");JLabel lbinarybackwards = new JLabel(" Binary Backward:  ");
			JLabel lleadingone = new JLabel(" Leading One:  ");JLabel lasciilength = new JLabel(" ASCII Length:  ");
			JLabel lbinarylength = new JLabel(" Binary Length:  ");JLabel llrcfound = new JLabel(" LRC Found:  ");
			JLabel llrcpassed = new JLabel(" LRC Passed:  ");JLabel lpeaksrequired = new JLabel(" Peaks Required:  ");
			lbpc.setToolTipText("Number Of Bits Used To Store Each Character");tbpc.setToolTipText("Number Of Bits Used To Store Each Character");
			lstartfound.setToolTipText("Displays If A Start Sentinel Was Detected");tstartfound.setToolTipText("Displays If A Start Sentinel Was Detected");
			lendfound.setToolTipText("Displays If An End Sentinel Was Detected");tendfound.setToolTipText("Displays If An End Sentinel Was Detected");
			lleadingzeros.setToolTipText("Number Of Zero Bits Before Start Sentinel");tleadingzeros.setToolTipText("Number Of Zero Bits Before Start Sentinel");
			ltrailingzeros.setToolTipText("Number Of Zero Bits After The LRC Character");ttrailingzeros.setToolTipText("Number Of Zero Bits After The LRC Character");
			lbinarybackwards.setToolTipText("Displays If The Binary Data Is Backwards");tbinarybackwards.setToolTipText("Displays If The Binary Data Is Backwards");
			lleadingone.setToolTipText("Displays If There Was A Non Standard Leading One Bit");tleadingone.setToolTipText("Displays If There Was A Non Standard Leading One Bit");
			lasciilength.setToolTipText("The Number Of ASCII Characters");tasciilength.setToolTipText("The Number Of ASCII Characters");
			lbinarylength.setToolTipText("The Number Of Binary Bits");tbinarylength.setToolTipText("The Number Of Binary Bits");
			llrcfound.setToolTipText("Displays If The LRC Character Was Detected");tlrcfound.setToolTipText("Displays If The LRC Character Was Detected");
			llrcpassed.setToolTipText("Displays If the LRC Character Was Valid");tlrcpassed.setToolTipText("Displays If the LRC Character Was Valid");
			lpeaksrequired.setToolTipText("The Number Of Peaks Required To Encode In FSK");tpeaksrequired.setToolTipText("The Number Of Peaks Required To Encode In FSK");	
			lbpc.setOpaque(true);lendfound.setOpaque(true);ltrailingzeros.setOpaque(true);//required for background coloring
			lleadingone.setOpaque(true);lbinarylength.setOpaque(true);llrcpassed.setOpaque(true);
			tbpc.setOpaque(true);tendfound.setOpaque(true);ttrailingzeros.setOpaque(true);
			tleadingone.setOpaque(true);tbinarylength.setOpaque(true);tlrcpassed.setOpaque(true);
			Color shaded = new Color(210,210,210);
			lbpc.setBackground(shaded);tbpc.setBackground(shaded);
			lendfound.setBackground(shaded);tendfound.setBackground(shaded);
			ltrailingzeros.setBackground(shaded);ttrailingzeros.setBackground(shaded);
			lleadingone.setBackground(shaded);tleadingone.setBackground(shaded);
			lbinarylength.setBackground(shaded);tbinarylength.setBackground(shaded);
			llrcpassed.setBackground(shaded);tlrcpassed.setBackground(shaded);
			
			gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 2;stats.add(lbpc,gbc);gbc.gridx = 2;gbc.gridy = 0;gbc.gridwidth = 1;stats.add(tbpc,gbc);
			gbc.gridx = 0;gbc.gridy = 1;gbc.gridwidth = 2;stats.add(lstartfound,gbc);gbc.gridx = 2;gbc.gridy = 1;gbc.gridwidth = 1;stats.add(tstartfound,gbc);
			gbc.gridx = 0;gbc.gridy = 2;gbc.gridwidth = 2;stats.add(lendfound,gbc);gbc.gridx = 2;gbc.gridy = 2;gbc.gridwidth = 1;stats.add(tendfound,gbc);
			gbc.gridx = 0;gbc.gridy = 3;gbc.gridwidth = 2;stats.add(lleadingzeros,gbc);gbc.gridx = 2;gbc.gridy = 3;gbc.gridwidth = 1;stats.add(tleadingzeros,gbc);
			gbc.gridx = 0;gbc.gridy = 4;gbc.gridwidth = 2;stats.add(ltrailingzeros,gbc);gbc.gridx = 2;gbc.gridy = 4;gbc.gridwidth = 1;stats.add(ttrailingzeros,gbc);
			gbc.gridx = 0;gbc.gridy = 5;gbc.gridwidth = 2;stats.add(lbinarybackwards,gbc);gbc.gridx = 2;gbc.gridy = 5;gbc.gridwidth = 1;stats.add(tbinarybackwards,gbc);
			gbc.gridx = 0;gbc.gridy = 6;gbc.gridwidth = 2;stats.add(lleadingone,gbc);gbc.gridx = 2;gbc.gridy = 6;gbc.gridwidth = 1;stats.add(tleadingone,gbc);
			gbc.gridx = 0;gbc.gridy = 7;gbc.gridwidth = 2;stats.add(lasciilength,gbc);gbc.gridx = 2;gbc.gridy = 7;gbc.gridwidth = 1;stats.add(tasciilength,gbc);
			gbc.gridx = 0;gbc.gridy = 8;gbc.gridwidth = 2;stats.add(lbinarylength,gbc);gbc.gridx = 2;gbc.gridy = 8;gbc.gridwidth = 1;stats.add(tbinarylength,gbc);
			gbc.gridx = 0;gbc.gridy = 9;gbc.gridwidth = 2;stats.add(llrcfound,gbc);gbc.gridx = 2;gbc.gridy = 9;gbc.gridwidth = 1;stats.add(tlrcfound,gbc);
			gbc.gridx = 0;gbc.gridy = 10;gbc.gridwidth = 2;stats.add(llrcpassed,gbc);gbc.gridx = 2;gbc.gridy = 10;gbc.gridwidth = 1;stats.add(tlrcpassed,gbc);
			gbc.gridx = 0;gbc.gridy = 11;gbc.gridwidth = 2;stats.add(lpeaksrequired,gbc);gbc.gridx = 2;gbc.gridy = 11;gbc.gridwidth = 1;stats.add(tpeaksrequired,gbc);
			
			side.add(options);
			side.add(stats);
			side.add(Box.createGlue());
			jdPanel.add(side, BorderLayout.EAST);
			jd.setContentPane(jdPanel);	
			jd.pack();
			jd.setMinimumSize(new Dimension(500,350));
			Dimension d = view.getSize();
			Point p = view.getLocationOnScreen();
			jd.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
			jd.setVisible(true);
		}
		
		private class conversionActionListener extends KeyAdapter implements ActionListener {
			int bpc;
			conversionActionListener(){
				bpc = 0;
			}
			
			public void actionPerformed(ActionEvent arg0) {
				Object button = arg0.getSource();
				if(button.getClass() == JRadioButton.class && ((JRadioButton)button).getText() == " Auto "){
					bpc = 0;
				}else if(button.getClass() == JRadioButton.class && ((JRadioButton)button).getText() == " 5 "){
					bpc = 5;
				}else if(button.getClass() == JRadioButton.class && ((JRadioButton)button).getText() == " 7 "){
					bpc = 7;
				}else if(button.getClass() == JButton.class && ((JButton)button).getText() == "ASCII -> Binary "){
					binary.setText(atob(ascii.getText()));
				}else if(button.getClass() == JButton.class && ((JButton)button).getText() == "Binary -> ASCII "){
					ascii.setText(btoa(binary.getText()));
				}
			}
			
			private String btoa(String binarystr){
				//TODO shade the crc to show failures
				String retval = "";
				if(binarystr.length() == 0){
					JOptionPane.showMessageDialog(jd, "Binary Box Did Not Contain Any Characters", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					return retval;
				}else if(! binarystr.matches("^[01]*$")){
					JOptionPane.showMessageDialog(jd, "Binary Box Can Only Contain a '0' or a '1'", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					return retval;
				}else if(binarystr.matches("^0*$")){
					JOptionPane.showMessageDialog(jd, "Binary Box Contained All 0's\nCan Not Decode", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					return retval;
				}else if(binarystr.length() > 8 && binarystr.startsWith("1") && binarystr.substring(1,6).equals("0000000")){
					binarystr = binarystr.substring(1);
					//TODO set initial 1 bit, technically we should only need 00 since start sentinels are 11010 and 1010...
				}
				//the initial checking is done above but here is where you need to take binary and make it ascii
			
				retval = Swipe.decodeBin(binarystr);
				if(retval == null) {
				    retval = Swipe.decodeBin((new StringBuffer(binarystr)).reverse().toString());
				}				
				
				return retval;
			}
			
			private String atob(String asciistr){
				String retval = "";
				if(asciistr.length() == 0){
					JOptionPane.showMessageDialog(jd, "ASCII Box Did Not Contain Any Characters", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					return retval;
				}else if(bpc == 0 && asciistr.charAt(0) == ';' && asciistr.charAt(asciistr.length()-1) == '?'){
					retval = convAToB(asciistr, 5);
					
				}else if(bpc == 0 && asciistr.charAt(0) == '%' && asciistr.charAt(asciistr.length()-1) == '?'){
					for(int i=0;i<26;i++){//replace lowercase with uppercase
						asciistr = asciistr.replace(((char)(i+97)), ((char)(i+65)));
					}
					ascii.setText(asciistr);
					retval = convAToB(asciistr, 7);
					
				}else if(bpc == 0){//has to be after the 5/7 bit ss/es checks
					JOptionPane.showMessageDialog(jd, "Couldn't Find Start And End Sentinels To Determine Bits Per Character\nEither Statically Set The Bits Per Character, Or Add Start And End Sentinels", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					return retval;
				}else if(bpc == 5){
					if(asciistr.matches("^[0-9;:<=>?]*$")){
						retval = convAToB(asciistr, bpc);
					}else{//this should happen due to string verifier
						JOptionPane.showMessageDialog(jd, "With 5 bits per char, ASCII Text Must Only Contain:\n0-9:;<=>?\nLine Breaks Are Also Not Allowed", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					}
					
				}else if(bpc == 7){
					if(asciistr.matches("^[A-Za-z0-9 !\"#$%&\'()*+,-./:;<=>?@^_\\[\\]\\\\]*$")){
						for(int i=0;i<26;i++){//replace lowercase with uppercase
							asciistr = asciistr.replace(((char)(i+97)), ((char)(i+65)));
						}
						ascii.setText(asciistr);
						retval = convAToB(asciistr, bpc);
					}else{//this should happen due to string verifier
						JOptionPane.showMessageDialog(jd, "With 7 bits per char, ASCII Text Must Only Contain:\nA-Z a-z 0-9 !\"#$%&\'()*+,-./:;<=>?@^_[\\]*$\nLine Breaks Are Also Not Allowed", "Conversion Error", JOptionPane.WARNING_MESSAGE);
					}
				}
				
				return retval;
			}
			
			private String convAToB(String data, int abpc){
				String retdata = "";
				for(int i=0;i<data.length();i++){
					int charval = data.charAt(i) - (abpc == 7 ? 32 : 48);
					boolean crc = true;
					for(int j=0;j<abpc-1;j++){
						int bit = charval & (1 << j);//generate every bit for the string
						if( (bit & (1 << j)) != 0){
							crc = !crc;//flip crc if needed
							retdata += "1";
						}else
							retdata += "0";
					}
					retdata += crc ? "1" : "0";
				}
				retdata += genLRC(retdata, abpc);
				return retdata;
			}
			
			private String genLRC(String data, int bpc){
				String lrc = "";
				boolean crc = true;
				for(int i=0;i<bpc-1;i++){//lrc time
					boolean par = true;
					for(int k=0;k<(data.length()/bpc)-1;k++){
						if(data.charAt(i+bpc*k) == '1'){
							par = !par;
						}
					}
					if(par)//crc char for lrc
						crc = !crc;
					lrc += par ? "1" : "0";//tac on lrc
				}
				lrc += crc ? "1" : "0";
				return lrc;
			}
			
			private int peaksRequired(String arg){
				int retval = 1;
				for (int i=0;i<arg.length();i++)
					retval += arg.charAt(i) == '1' ? 2 : 1;
				return retval;
			}
			
			public void keyPressed(KeyEvent e){
				DefaultFocusManager focusManager = new DefaultFocusManager();
				if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
					jd.dispose();
				else if(e.getKeyCode() == KeyEvent.VK_DOWN)
					focusManager.focusNextComponent();
				else if(e.getKeyCode() == KeyEvent.VK_UP)
					focusManager.focusPreviousComponent();
				else if(e.getKeyCode() == KeyEvent.VK_ENTER && e.getSource().getClass() == JButton.class)
					((JButton)e.getSource()).doClick();
			}
			
//			private void formatBinary(){
//				StyledDocument binarysd = binary.getStyledDocument();
//				addStyles(binarysd);
//				String strippedbin = binary.getText();
//				short strippedbinlen = (short) strippedbin.length();
//				if(bpc > 0){
//					String[] binaryst = new String[strippedbinlen*2/bpc];
//					for(int i=0;i<strippedbinlen/bpc;i++){
//						binaryst[i*2] = strippedbin.substring(i*bpc,i*bpc+bpc-1);
//						binaryst[i*2+1] = strippedbin.substring(i*bpc+bpc-1,i*bpc+bpc);
//					}
//					String[] binarystyle = new String[strippedbinlen*2/bpc];
//					for(int i=0;i<strippedbinlen*2/bpc;i++){
//						if(selected*2 != i && selected*2+1 != i){ //if this is not the selected char
//							if(i%4 == 0){  binarystyle[i] = "normal";//non crc, white background
//							}else if(i%4 == 1){  binarystyle[i] = "lightnormal";//crc, white background
//							}else if(i%4 == 2){  binarystyle[i] = "greyback";//non crc grey background
//							}else{  binarystyle[i] = "lightgreyback";//crc grey background
//							}
//						}else{ //it is the selected char
//							if(i%4 == 0){  binarystyle[i] = "boldnormal";//non crc, white background
//							}else if(i%4 == 1){  binarystyle[i] = "boldlightnormal";//crc, white background
//							}else if(i%4 == 2){  binarystyle[i] = "boldgreyback";//non crc, grey background
//							}else{	binarystyle[i] = "boldlightgreyback";//crc grey background
//							}
//						}		
//					}
//					StyledDocument sd = binary.getStyledDocument();
//					addStyles(sd);
//					for(int i=0;i<binaryst.length;i++){
//						try {
//							sd.insertString(sd.getLength(), binaryst[i], sd.getStyle(binarystyle[i]));
//						} catch (BadLocationException e) {
//							model.getLog().addMsg("Error: BadLocationException Caught When Trying To Update Converter Binary Data");
//							return;
//						}
//					}
//				}
				
//			}
			
			private void addStyles(StyledDocument doc){
				try {
					doc.remove(0, doc.getLength());
				} catch (BadLocationException e) {
					model.getLog().addMsg("Error: Error Removing Old Styled Document For 'Binary' Text Box");
					return;
				}
				Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
				Style normal = doc.addStyle("normal", def);
				StyleConstants.setFontFamily(def, "Monospaced");
				
				Style lightnormal = doc.addStyle("lightnormal", normal);//crc
				StyleConstants.setForeground(lightnormal, new Color(51,204,255));
				
				Style greyback = doc.addStyle("greyback", normal);//grey background
				StyleConstants.setBackground(greyback, new Color(221,221,221));
				
				Style boldnormal = doc.addStyle("boldnormal", normal); //selected
				StyleConstants.setBold(boldnormal, true);
				
				Style boldlightnormal = doc.addStyle("boldlightnormal", normal); //selected crc
				StyleConstants.setBold(boldlightnormal, true);
				StyleConstants.setForeground(boldlightnormal, new Color(51,204,255));
				
				Style boldgreyback = doc.addStyle("boldgreyback", normal); //selected grey background
				StyleConstants.setBold(boldgreyback, true);
				StyleConstants.setBackground(boldgreyback, new Color(221,221,221));
				
				Style boldlightgreyback = doc.addStyle("boldlightgreyback", normal); //selected crc grey background
				StyleConstants.setBold(boldlightgreyback, true);
				StyleConstants.setBackground(boldlightgreyback, new Color(221,221,221));
				StyleConstants.setForeground(boldlightgreyback, new Color(51,204,255));
				
				Style lightgreyback = doc.addStyle("lightgreyback", normal); //crc grey background
				StyleConstants.setBackground(lightgreyback, new Color(221,221,221));
				StyleConstants.setForeground(lightgreyback, new Color(51,204,255));
			}
			
		}//conversionactionlistener
		
		private class binaryVerifier extends InputVerifier{
			public boolean verify(JComponent input) {
				String txt = ((JTextPane)input).getText();
				if(txt.matches("^[01]*$") )
					return true;
				JOptionPane.showMessageDialog(jd, "Binary Text Must Only Contain \'0\' or \'1\'\nLine Breaks Are Also Not Allowed", "Input Error", JOptionPane.WARNING_MESSAGE);
				model.getLog().addMsg("Info: Invalid Binary Text Input Into Conversion Box");
				return false;
			}
		}
		private class asciiVerifier extends InputVerifier{
			public boolean verify(JComponent input){
				String txt = ((JTextPane)input).getText();
				if(txt.matches("^[A-Za-z0-9 !\"#$%&\'()*+,-./:;<=>?@^_\\[\\]\\\\]*$")) // '\\\\' escapes \, \\[ escapes [
					return true;
				JOptionPane.showMessageDialog(jd, "ASCII Text Must Only Contain:\nA-Z a-z 0-9 !\"#$%&\'()*+,-./:;<=>?@^_[\\]*$\nLine Breaks Are Also Not Allowed", "Input Error", JOptionPane.WARNING_MESSAGE);
				model.getLog().addMsg("Info: Invalid ASCII Text Input Into Conversion Box");
				return false;
			}
		}
	}
	
	private class characterTableListener extends KeyAdapter implements ActionListener{
		JDialog jd;
		
		characterTableListener(){
			JDialog.setDefaultLookAndFeelDecorated(true);
			jd = new JDialog(view, "Character Table", true);
			
		}
		
		private class characterTableModel7 extends AbstractTableModel {
			static final long serialVersionUID = 4362480612754L;
			private ArrayList<Object[]> data = new ArrayList<Object[]>(); //arraylist of the table data, each entry is a row
			private final String[] colnames = {"Char", "B0","B1","B2","B3","B4","B5","Par"};
			
			characterTableModel7(){
				Object[][] fulltable = {{"Space","0","0","0","0","0","0","1"},{"!","1","0","0","0","0","0","0"},{"\"","0","1","0","0","0","0","0"},{"#","1","1","0","0","0","0","1"},
				{"$","0","0","1","0","0","0","0"},{"% (SS)","1","0","1","0","0","0","1"},{"&","0","1","1","0","0","0","1"},	{"'","1","1","1","0","0","0","0"},
				{"(","0","0","0","1","0","0","0"},{")","1","0","0","1","0","0","1"},{"*","0","1","0","1","0","0","1"},{"+","1","1","0","1","0","0","0"},
				{",","0","0","1","1","0","0","1"},{"-","1","0","1","1","0","0","0"},{".","0","1","1","1","0","0","0"},{"/","1","1","1","1","0","0","1"},
				{"0","0","0","0","0","1","0","0"},{"1","1","0","0","0","1","0","1"},{"2","0","1","0","0","1","0","1"},{"3","1","1","0","0","1","0","0"},
				{"4","0","0","1","0","1","0","1"},{"5","1","0","1","0","1","0","0"},{"6","0","1","1","0","1","0","0"},{"7","1","1","1","0","1","0","1"},
				{"8","0","0","0","1","1","0","1"},{"9","1","0","0","1","1","0","0"},{":","0","1","0","1","1","0","0"},{";","1","1","0","1","1","0","1"},
				{"<","0","0","1","1","1","0","0"},{"=","1","0","1","1","1","0","1"},{">","0","1","1","1","1","0","1"},{"? (ES)","1","1","1","1","1","0","0"},
				{"@","0","0","0","0","0","1","0"},{"A","1","0","0","0","0","1","1"},{"B","0","1","0","0","0","1","1"},{"C","1","1","0","0","0","1","0"},
				{"D","0","0","1","0","0","1","1"},{"E","1","0","1","0","0","1","0"},{"F","0","1","1","0","0","1","0"},{"G","1","1","1","0","0","1","1"},
				{"H","0","0","0","1","0","1","1"},{"I","1","0","0","1","0","1","0"},{"J","0","1","0","1","0","1","0"},{"K","1","1","0","1","0","1","1"},
				{"L","0","0","1","1","0","1","0"},{"M","1","0","1","1","0","1","1"},{"N","0","1","1","1","0","1","1"},{"O","1","1","1","1","0","1","0"},
				{"P","0","0","0","0","1","1","1"},{"Q","1","0","0","0","1","1","0"},{"R","0","1","0","0","1","1","0"},{"S","1","1","0","0","1","1","1"},
				{"T","0","0","1","0","1","1","0"},{"U","1","0","1","0","1","1","1"},{"V","0","1","1","0","1","1","1"},{"W","1","1","1","0","1","1","0"},
				{"X","0","0","0","1","1","1","0"},{"Y","1","0","0","1","1","1","1"},{"Z","0","1","0","1","1","1","1"},{"[","1","1","0","1","1","1","0"},
				{"\\","0","0","1","1","1","1","1"},{"]","1","0","1","1","1","1","0"},{"^ (FS)","0","1","1","1","1","1","0"},{"_","1","1","1","1","1","1","1"}};
				for(int i=0;i<fulltable.length;i++){//add all that junk to the array
					data.add(fulltable[i]);
				}
			}
			public int getColumnCount() { return colnames.length; }

			public int getRowCount() { return data.size(); }

			public Object getValueAt(int row, int col) { return data.get(row)[col]; }
			
			public boolean isCellEditable(int r, int c){
				return false; //the real reason we're doing this abstracttablemodel crap
			}
			public String getColumnName(int i){ return colnames[i];	}
		}
		
		private class characterTableModel5 extends AbstractTableModel {
			static final long serialVersionUID = 4362480612753L;
			private ArrayList<Object[]> data = new ArrayList<Object[]>(); //arraylist of the table data, each entry is a row
			private final String[] colnames = {"Char", "B0","B1","B2","B3","Par"};
			characterTableModel5(){
				Object[][] fulltable = {{"0", "0", "0", "0", "0", "1"},{"1", "1", "0", "0", "0", "0"},{"2", "0", "1", "0", "0", "0"},
						{"3", "1", "1", "0", "0", "1"},{"4", "0", "0", "1", "0", "0"},{"5", "1", "0", "1", "0", "1"},
						{"6", "0", "1", "1", "0", "1"},{"7", "1", "1", "1", "0", "0"},{"8", "0", "0", "0", "1", "0"},
						{"9", "1", "0", "0", "1", "1"},{": (AS)", "0", "1", "0", "1", "1"},{"; (SS)", "1", "1", "0", "1", "0"},
						{"<", "0", "0", "1", "1", "1"},{"= (FS)", "1", "0", "1", "1", "0"},{">", "0", "1", "1", "1", "0"},
						{"? (ES)", "1", "1", "1", "1", "1"}};
					
				for(int i=0;i<fulltable.length;i++){//add all that junk to the array
					data.add(fulltable[i]);
				}
			}
			public int getColumnCount() {return colnames.length; }

			public int getRowCount() {return data.size(); }

			public Object getValueAt(int row, int col) {return data.get(row)[col]; }
			
			public boolean isCellEditable(int r, int c){
				return false; //the real reason we're doing this abstracttablemodel crap
			}
			public String getColumnName(int i){
				return colnames[i];
			}
		}
		
		public void actionPerformed(ActionEvent arg0) {
			
			//Track 2 and Track 3, 4 Bit ABA Character Table in Odd Parity
			//Track 1, 6 Bit IATA Character Table in Odd Parity
			JTable fivetable = new JTable() {
				static final long serialVersionUID = 4362480612752L;
				public Component prepareRenderer(TableCellRenderer renderer, int row, int col){
					Component c = super.prepareRenderer(renderer, row, col); 
					if(row %4 >= 2 ) {
						c.setBackground(Color.LIGHT_GRAY); 
					} else { 
						c.setBackground(getBackground()); 
					} 
					return c; 
				} 
			};
			fivetable.setModel(new characterTableModel5());
			fivetable.addKeyListener(this);
			fivetable.getColumnModel().getColumn(0).setPreferredWidth(40);
			for (int i = 1; i < 6; i++) {
				fivetable.getColumnModel().getColumn(i).setPreferredWidth(12);
			}
			
			JTable seventable = new JTable() {
				static final long serialVersionUID = 4362480612752L;
				public Component prepareRenderer(TableCellRenderer renderer, int row, int col){
					Component c = super.prepareRenderer(renderer, row, col); 
					if(row %4 >= 2 ) {
						c.setBackground(Color.LIGHT_GRAY); 
					} else { 
						c.setBackground(getBackground()); 
					} 
					return c; 
				} 
			};
			seventable.setModel(new characterTableModel7());
			seventable.addKeyListener(this);
			seventable.getColumnModel().getColumn(0).setPreferredWidth(35);
			for (int i = 1; i < 8; i++) {
				seventable.getColumnModel().getColumn(i).setPreferredWidth(12);
			}
			JTabbedPane tabbedpane = new JTabbedPane();
			tabbedpane.addKeyListener(this);
			JLabel seventitle = new JLabel(" Track 1, 6 Bit (+Odd Parity) IATA Character Table ");
			JScrollPane sevenscroll = new JScrollPane(seventable);
			sevenscroll.addKeyListener(this);
			sevenscroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			sevenscroll.setPreferredSize(new Dimension(250,400));
			JPanel sevenpanel = new JPanel(new BorderLayout());
			sevenpanel.add(seventitle, BorderLayout.NORTH);
			sevenpanel.add(sevenscroll, BorderLayout.CENTER);
			tabbedpane.addTab("7 Bit Table", sevenpanel);
			JLabel fivetitle = new JLabel(" Track 2 and 3, 4 Bit (+Odd Parity) ABA Character Table ");
			JScrollPane fivescroll = new JScrollPane(fivetable);
			fivescroll.addKeyListener(this);
			fivescroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			fivescroll.setPreferredSize(new Dimension(250,400));
			JPanel fivepanel = new JPanel(new BorderLayout());
			fivepanel.add(fivetitle, BorderLayout.NORTH);
			fivepanel.add(fivescroll, BorderLayout.CENTER);
			tabbedpane.addTab("5 Bit Table", fivepanel);
			
			jd.add(tabbedpane);
			jd.pack();
			Dimension d = view.getSize();
			Point p = view.getLocationOnScreen();
			jd.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
			jd.setVisible(true);
		}
		
		public void keyPressed(KeyEvent e){
			if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
				jd.dispose();
		}
	}
	
	private class lockScreenListener extends KeyAdapter implements ActionListener{
		JDialog jd;
		JButton acceptb, cancelb;
		JPasswordField pass1, pass2;
		
		public void actionPerformed(ActionEvent arg0) {
			JDialog.setDefaultLookAndFeelDecorated(true);
			jd = new JDialog(view, "Lock Screen", true);
			acceptb = new JButton("Lock Screen");
			acceptb.addKeyListener(this);
			acceptb.addActionListener(new acceptListenerDialog());
			cancelb = new JButton("Cancel");
			cancelb.addKeyListener(this);
			cancelb.addActionListener(new cancelListenerDialog());
			JLabel text = new JLabel(" Enter A Password Twice, That Will Unlock The Screen  ");
			pass1 = new JPasswordField();
			pass1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Password"));
			pass1.setBackground(new Color(238,238,238));
			pass1.addKeyListener(this);
			pass2 = new JPasswordField();
			pass2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Password Again"));
			pass2.setBackground(new Color(238,238,238));
			pass2.addKeyListener(this);
			
			JPanel jp = new JPanel(new BorderLayout());
			JPanel center = new JPanel();
			center.setLayout(new BoxLayout(center, BoxLayout.Y_AXIS));
			center.add(text);
			center.add(pass1);
			center.add(pass2);
			JPanel bottom = new JPanel();
			bottom.setLayout(new BoxLayout(bottom, BoxLayout.X_AXIS));
			bottom.add(Box.createHorizontalGlue());
			bottom.add(acceptb);
			bottom.add(cancelb);
			
			jp.add(center, BorderLayout.CENTER);
			jp.add(bottom, BorderLayout.SOUTH);
			jd.setContentPane(jp);
			jd.pack();
			Dimension d = view.getSize();
			Point p = view.getLocationOnScreen();
			jd.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
			jd.setResizable(false);
			jd.setVisible(true);
		}
		
		private class acceptListenerDialog extends KeyAdapter implements ActionListener{
			JButton unlock;
			String hashedstring;
			JPasswordField pass;
			JDialog jd2;
			
			public void actionPerformed(ActionEvent arg0) {
				jd2 = new JDialog(view, "Screen Locked", true);
				if( ! String.valueOf( pass1.getPassword() ).equals( String.valueOf( pass2.getPassword()) ) ){
					JOptionPane.showMessageDialog(jd2, "Passwords Did Not Match", "Lock Screen Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Debug: Could Not Lock Screen, Passwords Did Not Match");
				}else if(pass1.getPassword().length == 0){
					JOptionPane.showMessageDialog(jd2, "Passwords Can't Be Blank", "Lock Screen Error", JOptionPane.WARNING_MESSAGE);
				}else{
					jd2.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
					MessageDigest digest = null; //calculate the hash
					try {
						digest = MessageDigest.getInstance("MD5");
					} catch (NoSuchAlgorithmException e1) {
						model.getLog().addMsg("Error: Could Not Load MD5 Hashing Function");
						return;
					}
					int salt = (int)(Math.random()*65536);
					byte[] suffix = String.valueOf(pass1.getPassword()).getBytes();
					byte[] plain = new byte[suffix.length+2];//prepend 2 bytes of salt
					System.arraycopy(suffix, 0, plain, 2, suffix.length);
					plain[0] = (byte)((salt & 0xff00)>>>8);//high order byte of salt, shift down to a byte and add it in the first position
					plain[1] = (byte)(salt & 0xff);//low order byte of salt, cut out 
					byte[] hashed = digest.digest(plain);//hash it
					hashedstring = "";
					for(int i=0;i<16;i++){
						int high = (hashed[i] & 0xf0)>>>4;//split up the current byte
						int low = (hashed[i]) & 0x0f;
						hashedstring += Integer.toString(high, 16);//append the nibble to the string in hex
						hashedstring += Integer.toString(low, 16);
					}
					
					pass1.setText("");//clear the password
					pass2.setText("");		
					JPanel main = new JPanel();
					main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
					JPanel bottom = new JPanel();
					bottom.setLayout(new BoxLayout(bottom, BoxLayout.X_AXIS));
					bottom.add(Box.createHorizontalGlue());
					unlock = new JButton("Unlock Screen");
					bottom.add(unlock);
					pass = new JPasswordField(26);
					pass.addKeyListener(this);
					unlock.addKeyListener(this);
					unlock.addActionListener(new unlockListenerDialog());
					pass.setBackground(new Color(238,238,238));
					pass.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Password"));
					main.add(pass);
					main.add(bottom);
					
					jd2.setContentPane(main);
					jd2.pack();
					Dimension d = view.getSize();
					Point p = view.getLocationOnScreen();
					jd2.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
					view.lockScreen(true);//remove the tabs so info isnt leaked
					jd.dispose();//has to be before jd2 visible
					jd2.setVisible(true);
					
				}
			}
			
			public void keyPressed(KeyEvent e){
				DefaultFocusManager focusManager = new DefaultFocusManager();
				if(e.getKeyCode() == KeyEvent.VK_DOWN)
					focusManager.focusNextComponent();
				else if(e.getKeyCode() == KeyEvent.VK_UP)
					focusManager.focusPreviousComponent();
				else if(e.getKeyCode() == KeyEvent.VK_ENTER)
					unlock.doClick();
			}
			
			private class unlockListenerDialog implements ActionListener {

				public void actionPerformed(ActionEvent arg0) {				
					byte[] suffix = String.valueOf(pass.getPassword()).getBytes();//plaintext data
					byte[] plain = new byte[suffix.length+2];//add two places in the array for the salt
					System.arraycopy(suffix, 0, plain, 2, suffix.length); //copy the array with two salt addresses @ 0-1
					MessageDigest digest = null; //calculate the hash
					try {
						digest = MessageDigest.getInstance("MD5");
					} catch (NoSuchAlgorithmException e1) {
						model.getLog().addMsg("Error: Could Not Load MD5 Hashing Function");
						return;
					}
					
					boolean valid = false;
					for(int salt=0;salt<=65535;salt++){
						plain[0] = (byte)((salt & 0xff00)>>>8);//high order byte of salt, shift down to a byte and add it in the first position
						plain[1] = (byte)(salt & 0xff);//low order byte of salt, cut out 
						byte[] salttry = digest.digest(plain);
						
						for(int k=0;k<16;k++){
							char char1 = (Integer.toString( ((salttry[k] & 0xf0)>>>4), 16)).charAt(0);//high order char
							char char2 = (Integer.toString( (salttry[k] & 0x0f), 16)).charAt(0);//low order char
							if(char1 != hashedstring.charAt(2*k) || char2 != hashedstring.charAt(2*k+1)){//short circuit the checking
								break;
							}else if(k==15){//passed
								valid = true;
								break;
							}
						}
					}
					
					if(valid){
						view.lockScreen(false);
						jd2.dispose();
					}else{
						pass.setText("");
						JOptionPane.showMessageDialog(jd2, "Incorrect Password", "Unlock Screen Error", JOptionPane.WARNING_MESSAGE);
					}
				}
			}
		}
		
		public void keyPressed(KeyEvent e){
			DefaultFocusManager focusManager = new DefaultFocusManager();
			if(e.getKeyCode() == KeyEvent.VK_DOWN)
				focusManager.focusNextComponent();
			else if(e.getKeyCode() == KeyEvent.VK_UP)
				focusManager.focusPreviousComponent();
			else if(e.getKeyCode() == KeyEvent.VK_ENTER)
				acceptb.doClick();
			else if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
				jd.dispose();
		}
		
		/**
		 * Listener called when the cancel button is pressed for this dialog
		 */
		private class cancelListenerDialog implements ActionListener{//cancel button displayed in dialog for adduser
			/**
			 * Method called when the cancel button is pressed for this dialog
			 */
			public void actionPerformed(ActionEvent e){
				jd.dispose();
			}
		}
	}
	
	/**
	 * Listener called when the preferences menu is selected
	 */
	private class prefListener extends KeyAdapter implements ActionListener{
		JDialog jd;
		JTextField zerolvlt, micthrest, defdoort, numswipest, saltRanget;
		JComboBox spacechart, comportt;
		JCheckBox zerocheck, secLogCheck;
		JButton acceptb;
		
		/**
		 * Listener called when a key is pressed, used for enter/esc/up/down shortcuts from any input box
		 */
		public void keyPressed(KeyEvent e){
			DefaultFocusManager focusManager = new DefaultFocusManager();
			if(e.getKeyCode() == KeyEvent.VK_DOWN)
				focusManager.focusNextComponent();
			else if(e.getKeyCode() == KeyEvent.VK_UP)
				focusManager.focusPreviousComponent();
			else if(e.getKeyCode() == KeyEvent.VK_ENTER)
				acceptb.doClick();
			else if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
				jd.dispose();
		}
		
		/**
		 * Listener called when the preferences menu is selected
		 */
		public void actionPerformed(ActionEvent e){
			JDialog.setDefaultLookAndFeelDecorated(true);
			jd = new JDialog(view, "Magstripper Preferences", true);
			JPanel jdPanel = new JPanel(new BorderLayout());
			JPanel jdCenter = new JPanel(new GridBagLayout());
			JLabel zeroLvl = new JLabel("  Static Zero Level ");
			zeroLvl.setToolTipText("Statically Set The Microphone Zero Level, Otherwise It Will Be Automatically Calculated");
			JLabel micThres = new JLabel("  Microphone Thresholds ");
			micThres.setToolTipText("The Distance From The 'Zero Level' Where A Swipe Will Be Detected");
			JLabel defDoor = new JLabel("  Default Unlock Time ");
			defDoor.setToolTipText("The Default Time In Milliseconds That The Strike Will Be Unlocked");
			JLabel numSwipes = new JLabel("  Swipes To Store ");
			numSwipes.setToolTipText("Only Store This Many Swipes In The Reader List, Set To 0 For Infinite");
			JLabel spaceChar = new JLabel("  Space Character ");
			spaceChar.setToolTipText("Replace All Spaces In The Decoded String With Another Character");
			JLabel comPort = new JLabel("  Com Port ");
			comPort.setToolTipText("Com Port That The Strike Is Connected To");
			JLabel saltRange = new JLabel("  Crypto Salt Range ");
			saltRange.setToolTipText("The Range Of Possible Salt Values (0-N) To Prepend To The Plaintext Data Before Hashing");
			JLabel secLog = new JLabel("  Secure Log ");
			secLog.setToolTipText("Don't Display Plain Text Data Before Hashing In The System Console Debug Log");
			
			zerocheck = new JCheckBox();
			zerocheck.addKeyListener(this);
			zerocheck.setSelected(!model.getZeroAutomatic()); //not checked = not enabled
			zerocheck.addActionListener(new zeroCheckListenerDialog());
			zerocheck.setToolTipText("Statically Set The Microphone Zero Level, Otherwise It Will Be Automatically Calculated");
			zerolvlt = new JTextField(12);
			zerolvlt.addKeyListener(this);
			zerolvlt.setToolTipText("Statically Set The Microphone Zero Level, Otherwise It Will Be Automatically Calculated");
			zerolvlt.setText(zerocheck.isSelected() ? model.getZeroLevel()+"" : "Automatic");
			zerolvlt.setEnabled(zerocheck.isSelected());
			JPanel zeroInput = new JPanel();zeroInput.setLayout(new BoxLayout(zeroInput, BoxLayout.X_AXIS));
			zeroInput.add(zerocheck);zeroInput.add(zerolvlt);
			micthrest = new JTextField(12);
			micthrest.addKeyListener(this);
			micthrest.setToolTipText("The Distance From The 'Zero Level' Where A Swipe Will Be Detected");
			micthrest.setText(model.getDelta()+"");
			defdoort = new JTextField(12);
			defdoort.addKeyListener(this);
			defdoort.setToolTipText("The Default Time In Milliseconds That The Strike Will Be Unlocked");
			defdoort.setText(model.getLockTime()+"");
			numswipest = new JTextField(12);
			numswipest.addKeyListener(this);
			numswipest.setToolTipText("Only Store This Many Swipes In The Reader List, Set To 0 For Infinite");
			numswipest.setText(model.getNumSwipes()+"");
			String[] opts = {"Space  ' '", "Tilde  '~'", "Section Symbol  ''", "Currency Symbol  ''"}; 
			spacechart = new JComboBox(opts);
			spacechart.setPreferredSize(new Dimension(80,21));
			spacechart.addKeyListener(this);
			spacechart.setToolTipText("Replace All Spaces In The Decoded String With Another Character");
			if(model.getSpaceChar() == '~') spacechart.setSelectedIndex(1);
			if(model.getSpaceChar() == '') spacechart.setSelectedIndex(2);
			if(model.getSpaceChar() == '') spacechart.setSelectedIndex(3);
			String[] opts2 = {"COM1", "COM2", "COM3", "COM4"};
			comportt = new JComboBox(opts2);
			comportt.setPreferredSize(new Dimension(80,21));
			comportt.addKeyListener(this);
			comportt.setSelectedItem(model.getComPort());
			comportt.setToolTipText("Com Port That The Strike Is Connected To");
			saltRanget = new JTextField(12);
			saltRanget.addKeyListener(this);
			saltRanget.setToolTipText("The Range Of Possible Salt Values (0-N) To Prepend To The Plaintext Data Before Hashing");
			saltRanget.setText(model.getSaltRange()+"");
			secLogCheck = new JCheckBox();
			secLogCheck.addKeyListener(this);
			secLogCheck.setSelected(model.getSecLog());
			secLogCheck.setToolTipText("Don't Display Plain Text Data Before Hashing In The System Console Debug Log");
			
			GridBagConstraints gbc = new GridBagConstraints();gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 1;jdCenter.add(zeroLvl, gbc);
			gbc.gridy = 1;jdCenter.add(micThres, gbc);
			gbc.gridy = 2;jdCenter.add(defDoor, gbc);
			gbc.gridy = 3;jdCenter.add(numSwipes, gbc);
			gbc.gridy = 4;jdCenter.add(spaceChar, gbc);
			gbc.gridy = 5;jdCenter.add(comPort, gbc);
			gbc.gridy = 6;jdCenter.add(saltRange, gbc);
			gbc.gridy = 7;jdCenter.add(secLog, gbc);
			gbc.gridy = 8;jdCenter.add(new JLabel(" "), gbc);
			gbc.gridx = 1;gbc.gridy = 0;jdCenter.add(zeroInput, gbc);
			gbc.gridy = 1;jdCenter.add(micthrest, gbc);
			gbc.gridy = 2;jdCenter.add(defdoort, gbc);
			gbc.gridy = 3;jdCenter.add(numswipest, gbc);
			gbc.gridy = 4;jdCenter.add(spacechart, gbc);
			gbc.gridy = 5;jdCenter.add(comportt, gbc);
			gbc.gridy = 6;jdCenter.add(saltRanget, gbc);
			gbc.gridy = 7;jdCenter.add(secLogCheck, gbc);
			
			acceptb = new JButton("Accept");
			acceptb.addKeyListener(this);
			JButton cancelb = new JButton("Cancel");
			cancelb.addKeyListener(this);
			JPanel buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			buttonPanel.add(Box.createHorizontalGlue());
			buttonPanel.add(acceptb);buttonPanel.add(Box.createRigidArea(new Dimension(5,5)));buttonPanel.add(cancelb);
			acceptb.addActionListener(new acceptListenerDialog());
			cancelb.addActionListener(new cancelListenerDialog());
			
			jdPanel.add(jdCenter, BorderLayout.CENTER);
			jdPanel.add(buttonPanel, BorderLayout.SOUTH);
			jd.setContentPane(jdPanel);	
			jd.pack();
			jd.setResizable(false);
			Dimension d = view.getSize();
			Point p = view.getLocationOnScreen();
			jd.setLocation(d.width/2+p.x-jd.getWidth()/2,d.height/2+p.y-jd.getHeight()/2);
			jd.setVisible(true);
		}
		
		/**
		 * Listener called when the checkbox for a static zero level is selected. This sets the field to editable or non editable (automatic)
		 */
		private class zeroCheckListenerDialog implements ActionListener{
			
			/**
			 * Method called when the check box is toggled
			 */
			public void actionPerformed(ActionEvent e){
				if(zerocheck.isSelected()){ //static zero level (box checked)
					zerolvlt.setEnabled(true);
					zerolvlt.setText(model.getZeroLevel()+"");
				}else{ //automatic zero level
					zerolvlt.setEnabled(false);
					zerolvlt.setText("Automatic");
				}
			}
		}
		
		/**
		 * Listener called when the accept button is pressed. All user input is verified and parsed.
		 */
		private class acceptListenerDialog implements ActionListener{
			
			/**
			 * Method called when the accept button is pressed
			 */
			public void actionPerformed(ActionEvent e){
				JDialog.setDefaultLookAndFeelDecorated(true);
				//try/catch each input to make sure we can at least convert it to a int, otherwise it's way outside of the range expected
				if(zerocheck.isSelected()){
					try{
						new Integer(zerolvlt.getText());
					}catch(NumberFormatException nfe){
						JOptionPane.showMessageDialog(jd, "Could Not Parse Zero Level, Must Be Between -32768 - 32767", "Preferences Error", JOptionPane.WARNING_MESSAGE);
						model.getLog().addMsg("Info: Could Not Parse Zero Level, Must Be Between -32768 - 32767");
						zerolvlt.setText(model.getZeroLevel()+"");
						return;
					}
				}
				try{
					new Integer(micthrest.getText());
				}catch(NumberFormatException nfe){
					JOptionPane.showMessageDialog(jd, "Could Not Parse Mic Threshold, Must Be Between 10 - 32767", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Parse Mic Threshold, Must Be Between 10 - 32767");
					micthrest.setText(model.getDelta()+"");
					return;
				}
				try{
					new Integer(defdoort.getText());
				}catch(NumberFormatException nfe){
					JOptionPane.showMessageDialog(jd, "Could Not Parse Default Unlock Time, Must Be Between 100ms And 43200000ms (12hr)", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Parse Default Unlock Time, Must Be Between 100ms And 43200000ms (12hr)");
					defdoort.setText(model.getLockTime()+"");
					return;
				}
				try{
					new Integer(numswipest.getText());
				}catch(NumberFormatException nfe){
					JOptionPane.showMessageDialog(jd, "Could Not Parse Swipes To Store, Must Be Between 0 - 10000", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Parse Swipes To Store, Must Be Between 0 - 10000");
					numswipest.setText(model.getNumSwipes()+"");
					return;
				}
				try{
					new Integer(saltRanget.getText());
				}catch(NumberFormatException nfe){
					JOptionPane.showMessageDialog(jd, "Could Not Parse Salt Range To Store, Must Be Between 0 - 10000", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Parse Swipes To Store, Must Be Between 0 - 10000");
					saltRanget.setText(model.getSaltRange()+"");
					return;
				}
				
				//now we can at least parse the input data, do bounds checking
				if( new Integer(defdoort.getText()).intValue() > 43200000 || new Integer(defdoort.getText()).intValue() < 100 ){
					JOptionPane.showMessageDialog(jd, "Unlock Time Must Be Betweeen 100ms And 43200000ms (12hr)", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Change Preferences, Unlock Time not between 100 - 43200000ms");
					defdoort.setText(model.getLockTime()+"");
					return;
				}
				if( zerocheck.isSelected() && (new Integer(zerolvlt.getText()).intValue() < -32768 || new Integer(zerolvlt.getText()).intValue() > 32767) ){
					JOptionPane.showMessageDialog(jd, "Zero Level Must be betweeen -32768 and 32767", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Change Preferences, Zero Level Not Between -32768 - 32767");
					zerocheck.setSelected(model.getZeroAutomatic());
					zerolvlt.setText(model.getZeroLevel()+"");
					return;
				}
				if( zerocheck.isSelected() && (new Integer(zerolvlt.getText()).intValue() + new Integer(micthrest.getText()).intValue() > 32767 || new Integer(zerolvlt.getText()).intValue() - new Integer(micthrest.getText()).intValue() < -32768) ){
					JOptionPane.showMessageDialog(jd, "Zero Level +/- Threshold Must be betweeen -32768 and 32767", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Could Not Change Preferences, Zero Level +/- Threshold Not Between -32768 - 32767");
					zerocheck.setSelected(model.getZeroAutomatic());
					zerolvlt.setText(model.getZeroLevel()+"");
					return;
				}
				if( new Integer(micthrest.getText()).intValue() > 32767 || new Integer(micthrest.getText()).intValue() < 10 ){
					JOptionPane.showMessageDialog(jd, "Threshold Must be betweeen 10 and 32767", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Cound Not Change Preferences, Threshold Must Be Betweeen 10 And 32767");
					micthrest.setText(model.getDelta()+"");
					return;
				}
				if( new Integer(numswipest.getText()).intValue() > 10000 || new Integer(numswipest.getText()).intValue() < 0 ){
					JOptionPane.showMessageDialog(jd, "Swipes To Store Must be betweeen 0 and 10000", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Cound Not Change Preferences, Swipes To Store Must be betweeen 0 and 10000");
					numswipest.setText(model.getNumSwipes()+"");
					return;
				}
				if( new Integer(saltRanget.getText()).intValue() > 65535 || new Integer(saltRanget.getText()).intValue() < 0 ){
					JOptionPane.showMessageDialog(jd, "Crypto Salt Range Must be betweeen 0 and 65535", "Preferences Error", JOptionPane.WARNING_MESSAGE);
					model.getLog().addMsg("Info: Cound Not Change Preferences, Salt Range Must be betweeen 0 and 65535");
					saltRanget.setText(model.getSaltRange()+"");
					return;
				}
				
				//input checking done
				if(model.getLockTime() != new Integer(defdoort.getText()).intValue()){ //if the lock time does not equal the current lock time
					model.setLockTime(new Integer(defdoort.getText()).intValue());
				}
				if(zerocheck.isSelected() && new Integer(zerolvlt.getText()).intValue() != model.getZeroLevel()){ //if zero level is statically set, and it is not the current zero level
					model.setZeroAutomatic(false);
					model.setZeroLevel(new Integer(zerolvlt.getText()).intValue());
				}else if(!zerocheck.isSelected() && !model.getZeroAutomatic()) //if the prefs were set to automatic, and it wasnt previously set to automatic set it
					model.setZeroAutomatic(true);
				if(model.getDelta() != new Integer(micthrest.getText()).intValue())
					model.setDelta( (short)new Integer(micthrest.getText()).intValue() );
				if(model.getNumSwipes() != new Integer(numswipest.getText()).intValue()){
					model.setNumSwipes((short)new Integer(numswipest.getText()).intValue());
					while(model.getSwipeCount() > model.getNumSwipes() && model.getNumSwipes() != 0){//over the maximum swipes allowed remove one
						model.removeSwipe(view.getOldestSwipe());
						view.removeOldestSwipe();
					}
					view.setSwipeCount(model.getSwipeCount());
				}
				char selected;
				switch(spacechart.getSelectedIndex()){
					case 1: selected = '~';break;
					case 2: selected = '';break;
					case 3: selected = '';break;
					default: selected = ' ';break;
				}
				if(selected != model.getSpaceChar()){
					model.setSpaceChar(selected);
				}
				if(model.getComPort() != comportt.getSelectedItem())
					model.setComPort((String)comportt.getSelectedItem());
				if(model.getSaltRange() != new Integer(saltRanget.getText()).intValue()){
					if(model.getSaltRange() > new Integer(saltRanget.getText()).intValue()){
						Object[] opt = {"Continue", "Use Old Range"};
						if(1 == JOptionPane.showOptionDialog(view, "The New Salt Range Is Smaller Than It Was\nOld Card Hashes Might Not Work\nPreventing The Strike From Openeing", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1])){
							saltRanget.setText(model.getSaltRange()+"");
							return;
						}
					}
					model.setSaltRange(new Integer(saltRanget.getText()).intValue());
				}
				if(model.getSecLog() != secLogCheck.isSelected())
					model.setSecLog(secLogCheck.isSelected());
				jd.dispose();
			}
		}
		/**
		 * Listener called when the cancel button is pressed for this dialog
		 */
		private class cancelListenerDialog implements ActionListener{//cancel button displayed in dialog for adduser
			
			/**
			 * Method called when the cancel button is pressed for this dialog
			 */
			public void actionPerformed(ActionEvent e){
				jd.dispose();
			}
		}
	}
	
	/**
	 * Listener called when the close button is pressed, or exit is selected from the menu. This checks if there are swipes open, or data in the user table and warns the user that the data will be lost.
	 */
	private class exitListener implements WindowListener, ActionListener{ //catch the window closing event and make sure they really want to exit
		public void windowClosed(WindowEvent we){		}

		/**
		 * Method called when the menu item "exit" is selected, spawn window closing event.
		 */
		public void actionPerformed(ActionEvent e){
			windowClosing(new WindowEvent(view, WindowEvent.WINDOW_CLOSING));
		}
		/**
		 * Method called when the program is about to close
		 */
		public void windowClosing(WindowEvent we){
			JDialog.setDefaultLookAndFeelDecorated(true);
			if(model.getSwipeCount() > 0 && model.getTableModel().getRowsDefined() > 0 ){
				JDialog.setDefaultLookAndFeelDecorated(true);
				Object[] opt = {"Quit", "Don't Quit"};
				if(0 == JOptionPane.showOptionDialog(view, " You have Swipe And User Information Open\n Are You Sure You Want To Quit?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
					System.exit(0);
			}else if(model.getSwipeCount() > 0){ 
				JDialog.setDefaultLookAndFeelDecorated(true);
				Object[] opt = {"Quit", "Don't Quit"};
				if(0 == JOptionPane.showOptionDialog(view, " You have Swipe Information Open\n Are You Sure You Want To Quit?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
					System.exit(0);
			}else if(model.getTableModel().getRowsDefined() > 0){ 
				JDialog.setDefaultLookAndFeelDecorated(true);
				Object[] opt = {"Quit", "Don't Quit"};
				if(0 == JOptionPane.showOptionDialog(view, " You have User Information Open\n Are You Sure You Want To Quit?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
					System.exit(0);
			}else if(model.getSwipeCount() == 0 && model.getTableModel().getRowsDefined() == 0)
				System.exit(0);
			else
				System.exit(1);
		}
		public void windowOpened(WindowEvent we){		}//required to implement to implement windowlistener
		public void windowDeiconified(WindowEvent we){		}
		public void windowIconified(WindowEvent we){		}
		public void windowActivated(WindowEvent we){		}
		public void windowDeactivated(WindowEvent we){		}
	}
	
	/**
	 * Listener called when the save button or menu item is selected.
	 */
	private class saveListener implements ActionListener{
		final JFileChooser fc = new JFileChooser();
		File def = new File(".wav"); //default file to hint that it has to be a wav
		/**
		 * Constructor for things only executed once
		 */
		saveListener(){
			fc.addChoosableFileFilter(new WavFilter());
		}
		/**
		 * Method called when the save button or menu item is selected.
		 */
		public void actionPerformed(ActionEvent e){
			String selected = view.getSwipeSelected();
			JDialog.setDefaultLookAndFeelDecorated(true);
			if(selected == null){ //no swipe selected
				JOptionPane.showMessageDialog(view, "No Swipe Selected To Save", "Save Error", JOptionPane.WARNING_MESSAGE);
			}else{
				fc.setSelectedFile(def);
				int clicked = fc.showSaveDialog(view);
				if(clicked == JFileChooser.APPROVE_OPTION){ //which button was clicked
					File toSave = fc.getSelectedFile();
					if(toSave.exists()){ //file exists will overwrite with intervention
						JDialog.setDefaultLookAndFeelDecorated(true);
						Object[] opt = {"Overwrite It", "Cancel"};
						if(1 == JOptionPane.showOptionDialog(view, " File Exists, Overwrite It?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
							actionPerformed(null);
					}
					model.getLog().addMsg("Info: Saving Swipe To File "+toSave.getAbsolutePath());
					ByteArrayInputStream stoSave = new ByteArrayInputStream(model.getSwipe(selected).getStream()); //have to rebuild the audioinputstream from byte[] due to lack of reset()
					AudioInputStream aiss = new AudioInputStream(stoSave, model.getSwipe(selected).getRawFormat(), model.getSwipe(selected).getFrameLength());
					try {
						AudioSystem.write(aiss, AudioFileFormat.Type.WAVE, toSave);
					} catch (FileNotFoundException e1) {
						model.getLog().addMsg("Warning: FileNotFoundException Caught When Saving File "+fc.getName());
						JOptionPane.showMessageDialog(view, "Can Not Save The Specified File\nDouble Check Your Permissions", "File Save Error",JOptionPane.ERROR_MESSAGE);
						return;
					} catch (IOException e1) {
						model.getLog().addMsg("Error: Could Not Save Wav File "+toSave.getAbsolutePath());
						JOptionPane.showMessageDialog(view, "Can Not Save The Specified File\nI/O Exception Encountered", "File Save Error",JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
			}
		}
	}
	
	/**
	 * Listener called when the open button, or menu item is selected.
	 */
	private class openListener implements ActionListener, Observer{
		final JFileChooser fc = new JFileChooser();;
		
		/**
		 * Constructor for things only executed once
		 */
		openListener(){
			model.addObserver(this);
			fc.addChoosableFileFilter(new WavFilter());
		}
		
		/**
		 * Method called when the open button or menu item is selected
		 */
		public void actionPerformed(ActionEvent e){
			JDialog.setDefaultLookAndFeelDecorated(true);
			fc.setFileHidingEnabled(true);
			int clicked = fc.showOpenDialog(view);
			if(clicked == JFileChooser.APPROVE_OPTION){ //which button was clicked
				File f = fc.getSelectedFile();
				AudioInputStream ais = null;
				try {
					ais = AudioSystem.getAudioInputStream(f);
				} catch (UnsupportedAudioFileException ex) {
					model.getLog().addMsg("Error: UnsupportedAudioFileException thrown when trying to open "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Open The Specified File\nFile Format Must Be .Wav", "File Open Error",JOptionPane.ERROR_MESSAGE);
					return;
				} catch (IOException ex) {
					model.getLog().addMsg("Error: IOException thrown when trying to open "+f.getName());
					JOptionPane.showMessageDialog(view, "Can Not Open The Specified File\nI/O Problem Encountered", "File Open Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
				Swipe s = new Swipe(ais);
				model.getLog().addMsg("Info: Opening File "+ f.getAbsolutePath());
				model.addSwipe(s);
			}
		}
		
		/**
		 * Method called when a new swipe is added to the model, normally by the mic input or opened file
		 * (After it's added to the model)
		 */
		public void update(Observable obs, Object obj){
			if(obj.getClass() == Swipe.class){
				if(climode){
					cli.printSwipe((Swipe)obj);
					model.removeSwipe(((Swipe)obj).getTimestamp());
				}else{
					view.addSwipe((Swipe)obj);
					if(model.getSwipeCount() > model.getNumSwipes() && model.getNumSwipes() != 0){//over the maximum swipes allowed remove one
						String id = view.getOldestSwipe();
						view.removeOldestSwipe();
						model.removeSwipe(id);
					}
					view.setSwipeCount(model.getSwipeCount());
				}
			}
		}
	}
	
	/**
	 * Listener called when a new swipe is selected in the list
	 */
	private class swipeListSelectionListener implements ListSelectionListener, Observer{
		/**
		 * Constructor that adds an observer of the model
		 */
		public swipeListSelectionListener(){
			model.addObserver(this);
		}
		
		/**
		 * Method called when a new swipe is selected
		 */
		public void valueChanged(ListSelectionEvent e){
			if(!e.getValueIsAdjusting()){ //if the user is done selecting
				view.updateSwipeSelection((model.getSwipe(view.getSwipeSelected())));
				model.getLog().addMsg("Debug: Updating Swipe Selection");
			}
		}
		
		/**
		 * Method called when a new space character is set, updates the gui automatically
		 */
		public void update(Observable obs, Object obj){
			if( obj == "space" ){
				if(view.getSwipeSelected() != null)
					view.updateSwipeSelection((model.getSwipe(view.getSwipeSelected())));
			}
		}
	}
	
	/**
	 * Listener called when a new character is selected in the list
	 */
	private class charListSelectionListener implements ListSelectionListener{
		
		/**
		 * Method called when a new character is selected in the list
		 */
		public void valueChanged(ListSelectionEvent e){
			if(!e.getValueIsAdjusting()){ //if the user is done selecting
				model.getLog().addMsg("Debug: Updating Character List Selection");
				view.updateCharSelection((model.getSwipe(view.getSwipeSelected())));
			}
		}
	}
	
	/**
	 * Listener called when the swipe's combo box filer is changed
	 */
	private class consoleLogComboBoxListener implements ActionListener{ //changed log level in the console combo box
		/**
		 * Method called when the swipe's combo box filter is updated
		 */
		public void actionPerformed(ActionEvent e){
			//model.getLog().addMsg("Debug: Updating Console Filter");
			if(view.getSwipeSelected() != null)
				view.updateSwipeConsole((model.getSwipe(view.getSwipeSelected())));
		}
	}
	
	/**
	 * Listener called with the system combo box is updated
	 */
	private class systemLogComboBoxListener implements ActionListener, Observer{ //changed log level in system combo box
		
		/**
		 * Constructor that adds a listener to the model for updates
		 */
		public systemLogComboBoxListener(){
			model.getLog().addObserver(this);
		}
		
		/**
		 * Method called when the combo box is updated
		 */
		public void actionPerformed(ActionEvent e){
			//model.getLog().addMsg("Debug: Updating System Filter"); //not needed
			view.updateSystemConsole();
		}
		
		/**
		 * Method called when the error log has a new message
		 */
		public void update(Observable obs, Object obj){
			if(obj == "log")
				view.updateSystemConsole();
		}
	}
	
	/**
	 * Listener called when the lock state is changed, or the test button is pressed
	 */
	private class lockListener implements ActionListener, Observer{
		lockListener(){
			model.addObserver(this);
		}
		
		/**
		 * Method called when the test lock button is pressed
		 */
		public void actionPerformed(ActionEvent e){
			model.getLog().addMsg("Debug: Launching Thread To Unlock Strike");
			Strike temp = null;
			try{
				temp = new Strike(model);
			}catch(NoClassDefFoundError ncdfe){
				model.getLog().addMsg("Error: NoClassDefFoundError Exception Caught, Make Sure You Installed RXTXcomm Correctly");
				JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane.showMessageDialog(view, "RXTXcomm Is Not Installed Properly\nCan't Access Serial Port", "Com Port Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			temp.start();
		}
		
		/**
		 * Method called when the lock state changes
		 */
		public void update(Observable obs, Object obj){
			if(obj == "lock")
				view.setLocked(model.getLockState());
		}
	}
	
	/**
	 * Listener called when the about screen should be shown
	 */
	private class aboutListener implements ActionListener{
		
		/**
		 * Method called when the about screen should be shown
		 */
		public void actionPerformed(ActionEvent e){
			model.getLog().addMsg("Debug: Displaying About Screen");
			view.updateSystemConsole();
			view.displayAbout();
		}
	}
	
	/**
	 * Filter used for save/load dialog boxes
	 */
	private class WavFilter extends FileFilter{//.wav filter for open/save dialogs
		/**
		 * Method required by FileFilter
		 */
		public boolean accept(File f){
			if(f.isDirectory()){ return true; }
			int dot = f.getName().lastIndexOf('.');
			if(dot > 0 && f.getName().substring(dot+1).toLowerCase().equals("wav")){ return true; }
			return false;
		}
		
		/**
		 * Returns a description of the file filter
		 */
		public String getDescription(){
			return "Wav Files";
		}
	}
	
	/**
	 * Filter used to save/load table files
	 */
	private class MagstripperFilter extends FileFilter{ //.mag filter for open/save tables
		/**
		 * Method required by FileFilter
		 */
		public boolean accept(File f){
			if(f.isDirectory()){ return true; }
			int dot = f.getName().lastIndexOf('.');
			if(dot > 0 && f.getName().substring(dot+1).toLowerCase().equals("mag")){ return true; }
			return false;
		}
		
		/**
		 * Returns the description of the file filter
		 * @returns string of the description
		 */
		public String getDescription(){
			return "Magstripper Files";
		}
	}
	
	private class TextFilter extends FileFilter{
		/**
		 * Method required by FileFilter
		 */
		public boolean accept(File f){
			if(f.isDirectory()){ return true; }
			int dot = f.getName().lastIndexOf('.');
			if(dot > 0 && f.getName().substring(dot+1).toLowerCase().equals("txt")){ return true; }
			return false;
		}
		/**
		 * Returns the description of the file filter
		 * @returns string of the description
		 */
		public String getDescription(){
			return "Text Files";
		}
	}
	
	/**
	 * Listener used to set a tabs focus based on what event occured
	 */
	private class tableTabListener implements ActionListener{
		
		/**
		 * Method called when a button is pressed and a tab should change
		 */
		public void actionPerformed(ActionEvent a) {
			String srctxt = ((AbstractButton)a.getSource()).getText(); //get text of a menu item
			String srcttip = ((AbstractButton)a.getSource()).getToolTipText(); //get tooltip for toolbar buttons
			
			if(srcttip == "Open Swipe .wav File" || srctxt == "Open .wav" || srcttip == "Save Swipe .wav File" || srctxt == "Save .wav" || srcttip == "Save Swipe Log File")
				view.setTabSelected(MagstripperGUI.CARD_READER);
			else if(srctxt == "Add User" || srctxt == "Delete User" || srctxt == "Open Table" || srctxt == "Save Table" || srctxt == "Clear Table")
				view.setTabSelected(MagstripperGUI.USER_TABLE);
			else if(srctxt == "Save System Log" || srcttip == "Save System Log File")
				view.setTabSelected(MagstripperGUI.SYSTEM_LOG);
			
		}
	}
	
	/**
	 * Listener used to save the swipe's log out to a file
	 */
	private class saveConsoleLogListener implements ActionListener{
		final JFileChooser fc = new JFileChooser();
		File def = new File(".txt"); //default file to hint that it has to be a wav
		
		saveConsoleLogListener(){
			fc.addChoosableFileFilter(new TextFilter());
		}
		
		/**
		 * Method called when the user selects the save swipe log button
		 */
		public void actionPerformed(ActionEvent a) {
			String selected = view.getSwipeSelected();
			JDialog.setDefaultLookAndFeelDecorated(true);
			if(selected == null){ //no swipe selected
				JOptionPane.showMessageDialog(view, "No Swipe Selected To Save It's Log File", "Save Error", JOptionPane.WARNING_MESSAGE);
			}else{
				fc.setSelectedFile(def);
				int clicked = fc.showSaveDialog(view);
				if(clicked == JFileChooser.APPROVE_OPTION){ //which button was clicked
					File toSave = fc.getSelectedFile();
					if(toSave.exists()){ //file exists will overwrite with intervention
						JDialog.setDefaultLookAndFeelDecorated(true);
						Object[] opt = {"Overwrite It", "Cancel"};
						if(1 == JOptionPane.showOptionDialog(view, " File Exists, Overwrite It?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
							actionPerformed(null);
					}
					model.getLog().addMsg("Info: Saving Swipe Log To File "+toSave.getAbsolutePath());
					try {
						FileOutputStream fos = new FileOutputStream(toSave);
						PrintStream ps = new PrintStream(fos);
						String[] writeme = model.getSwipe(selected).getErrorLog().getEntries();
						for(int i=0;i<writeme.length;i++){
							ps.println(writeme[i].substring(0,writeme[i].indexOf('\n')));
						}
						ps.flush();
						ps.close();
						try {
							fos.close();
						} catch (IOException e) {
							model.getLog().addMsg("Error: IOException Caught When Trying To Close Log File");
						}
					} catch (FileNotFoundException e) {
						model.getLog().addMsg("Warning: FileNotFoundException Caught When Saving Log File "+fc.getName());
						JOptionPane.showMessageDialog(view, "Can Not Save The Specified File\nDouble Check Your Permissions", "File Save Error",JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
			}
		}
	}
	
	/**
	 * Listener used to save the system log out to a file
	 */
	private class saveSystemLogListener implements ActionListener{
		final JFileChooser fc = new JFileChooser();
		File def = new File(".txt"); //default file to hint that it has to be a wav
		
		saveSystemLogListener(){
			fc.addChoosableFileFilter(new TextFilter());
		}
		
		/**
		 * Method called when the user selects the save swipe log button
		 */
		public void actionPerformed(ActionEvent a) {
			JDialog.setDefaultLookAndFeelDecorated(true);
			fc.setSelectedFile(def);
			int clicked = fc.showSaveDialog(view);
			if(clicked == JFileChooser.APPROVE_OPTION){ //which button was clicked
				File toSave = fc.getSelectedFile();
				if(toSave.exists()){ //file exists will overwrite with intervention
					JDialog.setDefaultLookAndFeelDecorated(true);
					Object[] opt = {"Overwrite It", "Cancel"};
					if(1 == JOptionPane.showOptionDialog(view, " File Exists, Overwrite It?", "Are You Sure", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, opt, opt[1]))
						actionPerformed(null);
				}
				model.getLog().addMsg("Info: Saving Swipe Log To File "+toSave.getAbsolutePath());
				try {
					FileOutputStream fos = new FileOutputStream(toSave);
					PrintStream ps = new PrintStream(fos);
					String[] writeme = model.getLog().getEntries();
					for(int i=0;i<writeme.length;i++){
						ps.println(writeme[i].substring(0,writeme[i].indexOf('\n')));
					}
					ps.flush();
					ps.close();
					try {
						fos.close();
					} catch (IOException e) {
						model.getLog().addMsg("Error: IOException Caught When Trying To Close Log File");
					}
				} catch (FileNotFoundException e) {
					model.getLog().addMsg("Warning: FileNotFoundException Caught When Saving Log File "+fc.getName());
					JOptionPane.showMessageDialog(view, "Can Not Save The Specified File\nDouble Check Your Permissions", "File Save Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
		}
	}
	
	/**
	 * Listener used to set the tooltip text of column headers of the user table
	 */
	private class userTableTooltipListener extends MouseMotionAdapter{
		String[] txt = {"Autogenerated Unique Identification Number", "Enable / Disable This User And All Of Their Cards", "First Name Of This User", 
				"Last Name Of This User", "Number Of Cards Attached To This User", "Time In Milliseconds The Strike Will Be Unlocked For This User" };
		
		/**
		 * Event triggered when the mouse is over a column header
		 */
		public void mouseMoved(MouseEvent me) {
			JTableHeader tableheader = (JTableHeader)me.getSource();
            TableColumnModel colModel = ((JTable)tableheader.getTable()).getColumnModel();
            
            if(colModel.getColumnIndexAtX(me.getX()) >= 0){
            	tableheader.setToolTipText(txt[colModel.getColumnIndexAtX(me.getX())]);
            }
		}
	}
	
	private class userTablePopupListener extends JPanel implements MouseListener, ActionListener{
		static final long serialVersionUID = 4362480612753L;
		JPopupMenu popup;
		JMenuItem delUser, delCards, dupUser;
		String[] opts = {"Delete User And Cards","Delete All User's Cards","Duplicate User And Cards"};
		private static final int DEL_USER = 0;
		private static final int DEL_USER_CARDS = 1;
		private static final int DUP_USER_CARDS = 2;
		
		private userTablePopupListener(){
			popup = new JPopupMenu();
			delUser = new JMenuItem(opts[0]);
			delUser.addActionListener(this);
			popup.add(delUser);
			delCards = new JMenuItem(opts[1]);
			delCards.addActionListener(this);
			popup.add(delCards);
			popup.addSeparator();
			dupUser = new JMenuItem(opts[2]);
			dupUser.addActionListener(this);
			popup.add(dupUser);
			
		}
		public void mouseClicked(MouseEvent me) {}
		public void mouseEntered(MouseEvent me) {}
		public void mouseExited(MouseEvent me) {}
		public void mousePressed(MouseEvent me) {
			if(me.isPopupTrigger()){
				popup.show(me.getComponent(), me.getX(), me.getY());
				int row = ((JTable)me.getSource()).rowAtPoint(new Point(me.getX(),me.getY()));//highlight the row before showing the menu
				((JTable)me.getSource()).getSelectionModel().setSelectionInterval(row, row);
			}
		}
		public void mouseReleased(MouseEvent me) {
			if(me.isPopupTrigger()){
				popup.show(me.getComponent(), me.getX(), me.getY());
				int row = ((JTable)me.getSource()).rowAtPoint(new Point(me.getX(),me.getY()));//highlight the row before showing the menu
				((JTable)me.getSource()).getSelectionModel().setSelectionInterval(row, row);
			}
		}

		public void actionPerformed(ActionEvent a) {
			if(((JMenuItem)a.getSource()).getText() == opts[DEL_USER]){
				if(view.getUserTableRowSelected() == -1){//shouldnt ever happen, just in case
					JDialog.setDefaultLookAndFeelDecorated(true);
					JOptionPane.showMessageDialog(view, "Please Select A Single User For Deletion", "User Delete Error", JOptionPane.WARNING_MESSAGE);
				}else{ //delete the user
					if(model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID).getClass() == Integer.class){ //if it's a non empty row
						model.getLog().addMsg("Info: Deleting User: "+model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID));		
					}
					model.getCardTableModel().updateTable(-1); //clear the card table
					model.getTableModel().delRow(view.getUserTableRowSelected());//delete the row
					view.updateCardCount();
					view.updateUserCount(); //decrement the user count
				}
			}else if(((JMenuItem)a.getSource()).getText() == opts[DEL_USER_CARDS] && view.getUserTableRowSelected() < model.getTableModel().getRowsDefined() ){
				model.getCardTableModel().delAllCards((Integer)model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID));
				model.getCardTableModel().updateTable(-1);
				model.getLog().addMsg("Info: Deleting All Cards For User: "+model.getTableModel().getValueAt(view.getUserTableRowSelected(), UserTableModel.ID));
				view.updateCardCount();
			}else if(((JMenuItem)a.getSource()).getText() == opts[DUP_USER_CARDS] && view.getUserTableRowSelected() < model.getTableModel().getRowsDefined() ){
				model.getTableModel().duplicateUser(view.getUserTableRowSelected());
				view.updateCardCount();
				view.updateUserCount();
			}
		}
	}
	
	
	
	
	
}