/*
This file is part of Magstripper, Copyright 2006, 2007.

Magstripper is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Magstripper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package magstripper;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

public class MagstripperGUI extends JFrame{
	static final long serialVersionUID = 4362480612754L; //to make extending JFrame happy
	private ImageIcon micStart, micStop, micStatusRed, micStatusGreen, micPrefStart, micPrefStop;
	private JButton toolMic, toolOpen, toolSave, toolSwipe, toolSystem, toolPref, toolLock, toolConv, toolTable, 
		toolLockScreen, toolHelp, userAdd, userDel, cardAdd, cardDel, userSave, userOpen, userClear;
	private JMenuItem helpAbout, helpHelp, optionPref, optionConv, optionTable, optionLock, micPref, lockPref, fileExit, fileSave, 
		fileSaveSystem, fileSaveConsole, fileOpen, tableAdd, tableDel, tableSave, tableOpen, tableClear;
	private MagstripperModel mmodel;
	private DefaultListModel swipeListModel, charListModel;
	private JList swipeList, charList;
	private JLabel tchannels, tformat, tsampleRate, tframeLength, tbitsPerSample, tbitOrder, tzeroLevel, tpositiveThreshold, tnegativeThreshold, tpositivePeaks, tnegativePeaks, 
					ttotalPeaks, tleading, ttrailing, tbitsPerChar, tswipeReverse, tleadingOne, trawLength, micStatusLabel, userCount, cardCount, swipeCount, doorStatus;
	private JTextArea characters, lrc, crcerr, lrcerr;
	private JTextField cdebugNum, cinfoNum, cwarnNum, cerrorNum, debugNum, infoNum, warnNum, errorNum;
	private JTextPane ascii, binary, rawbinary, console, systemText;
	private JComboBox consoleLogCombo, systemLogCombo;
	private JTable table, cardTable;
	private JTabbedPane tabs;
	private StyledDocument sd;
	public static int CARD_READER = 0;
	public static int USER_TABLE = 1;
	public static int SYSTEM_LOG = 2;
	
	/**
	 * Constructor for the view, 
	 * @param m the model of this system
	 */
	public MagstripperGUI(MagstripperModel m){
		super("Magstripper " + m.getVersion());
		mmodel = m;
	}
	
	/**
	 * Creates the top menubar of the GUI
	 * @return the populated menubar
	 */
	private JMenuBar createMenu(){
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        JMenu optionMenu = new JMenu("Options");
        optionMenu.setMnemonic(KeyEvent.VK_O);
        JMenu tableMenu = new JMenu("Table");
        tableMenu.setMnemonic(KeyEvent.VK_T);
        JMenu helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(fileMenu);
        menuBar.add(optionMenu);
        menuBar.add(tableMenu);
        menuBar.add(helpMenu);
        
        fileOpen = new JMenuItem("Open .wav",new ImageIcon(MagstripperGUI.class.getResource("icons/folder_open_file16.png")));
       	fileOpen.setMnemonic(KeyEvent.VK_O);
        fileOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        fileMenu.add(fileOpen);
        fileSave = new JMenuItem("Save .wav",new ImageIcon(MagstripperGUI.class.getResource("icons/floppy_wav16.png")));
        fileSave.setMnemonic(KeyEvent.VK_S);
        fileSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        fileMenu.add(fileSave);
        fileMenu.addSeparator();
        fileSaveConsole = new JMenuItem("Save Swipe Log", new ImageIcon(MagstripperGUI.class.getResource("icons/document_log_wav16.png")));
        fileSaveConsole.setMnemonic(KeyEvent.VK_W);
        fileSaveConsole.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
        fileMenu.add(fileSaveConsole);
        fileSaveSystem = new JMenuItem("Save System Log", new ImageIcon(MagstripperGUI.class.getResource("icons/document_log16.png")));
        fileSaveSystem.setMnemonic(KeyEvent.VK_Y);
        fileSaveSystem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
        fileMenu.add(fileSaveSystem);
        fileMenu.addSeparator();
        fileExit = new JMenuItem("Exit");
        fileExit.setMnemonic(KeyEvent.VK_E);
        fileExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
        fileMenu.add(fileExit);
        optionPref = new JMenuItem("Preferences", new ImageIcon(MagstripperGUI.class.getResource("icons/gears_options16.png")));
        optionPref.setMnemonic(KeyEvent.VK_P);
        optionPref.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
        optionMenu.add(optionPref);
        optionMenu.addSeparator();
        micPrefStart = new ImageIcon(MagstripperGUI.class.getResource("icons/microphone_next_move16.png"));
        micPrefStop = new ImageIcon(MagstripperGUI.class.getResource("icons/error_x16.png"));
        micPref = new JMenuItem("Start/Stop Mic Input", micPrefStart);
        micPref.setMnemonic(KeyEvent.VK_M);
        micPref.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
        optionMenu.add(micPref);
        lockPref = new JMenuItem("Test Lock", new ImageIcon(MagstripperGUI.class.getResource("icons/key16.png")));
        lockPref.setMnemonic(KeyEvent.VK_L);
        lockPref.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
        optionMenu.add(lockPref);
        optionMenu.addSeparator();
        optionConv = new JMenuItem("Conversion Tool", new ImageIcon(MagstripperGUI.class.getResource("icons/reload16.png")));
        optionConv.setMnemonic(KeyEvent.VK_T);
        optionConv.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
        optionMenu.add(optionConv);
        optionTable = new JMenuItem("Character Table", new ImageIcon(MagstripperGUI.class.getResource("icons/calendar16.png")));
        optionTable.setMnemonic(KeyEvent.VK_H);
        optionTable.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
        optionMenu.add(optionTable);
        optionLock = new JMenuItem("Lock Screen", new ImageIcon(MagstripperGUI.class.getResource("icons/lockscreen16.png")));
        optionLock.setMnemonic(KeyEvent.VK_K);
        optionLock.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K, ActionEvent.CTRL_MASK));
        optionMenu.add(optionLock);
        tableAdd = new JMenuItem("Add User", new ImageIcon(MagstripperGUI.class.getResource("icons/address_next_move_add_insert16.png")));
        tableAdd.setMnemonic(KeyEvent.VK_A);
        tableAdd.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        tableMenu.add(tableAdd);
        tableDel = new JMenuItem("Delete User", new ImageIcon(MagstripperGUI.class.getResource("icons/addresss_delete16.png")));
        tableDel.setMnemonic(KeyEvent.VK_D);
        tableDel.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        tableMenu.add(tableDel);
        tableMenu.addSeparator();
        tableOpen = new JMenuItem("Open Table", new ImageIcon(MagstripperGUI.class.getResource("icons/folder_document16.png")));
        tableOpen.setMnemonic(KeyEvent.VK_E);
        tableOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        tableMenu.add(tableOpen);
        tableSave = new JMenuItem("Save Table", new ImageIcon(MagstripperGUI.class.getResource("icons/document_floppy_save16.png")));
        tableSave.setMnemonic(KeyEvent.VK_V);
        tableSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        tableMenu.add(tableSave);
        tableMenu.addSeparator();
        tableClear = new JMenuItem("Clear Table", new ImageIcon(MagstripperGUI.class.getResource("icons/clear_wipe_erase16.png")));
        tableClear.setMnemonic(KeyEvent.VK_C);
        tableClear.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        tableMenu.add(tableClear);
        helpHelp = new JMenuItem("Magstripper Help",KeyEvent.VK_F1);
        helpHelp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, ActionEvent.CTRL_MASK));
        //helpMenu.add(helpHelp);
        helpAbout = new JMenuItem("About",KeyEvent.VK_B);
        helpAbout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK));
        helpMenu.add(helpAbout);
        return menuBar;
	}
	
	/**
	 * Creates the toolbar
	 * @return the populated toolbar
	 */
	private JToolBar createToolBar(){
		Insets noborder = new Insets(0,0,0,0);
		JToolBar toolBar = new JToolBar("Main Toolbar");
		toolOpen = new JButton( new ImageIcon(MagstripperGUI.class.getResource("icons/folder_open_file.png")) );
		toolOpen.setToolTipText("Open Swipe .wav File");
		toolOpen.setMargin(noborder);
		toolBar.add(toolOpen);
		toolSave = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/floppy_wav.png")));
		toolSave.setToolTipText("Save Swipe .wav File");
		toolSave.setMargin(noborder);
		toolBar.add(toolSave);
		toolBar.addSeparator();
		toolSwipe = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/document_log_wav.png")));
		toolSwipe.setToolTipText("Save Swipe Log File");
		toolSwipe.setMargin(noborder);
		toolBar.add(toolSwipe);
		toolSystem = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/document_log.png")));
		toolSystem.setToolTipText("Save System Log File");
		toolSystem.setMargin(noborder);
		toolBar.add(toolSystem);
		toolBar.addSeparator();
		toolPref = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/gears_options.png")));
		toolPref.setToolTipText("Preferences");
		toolPref.setMargin(noborder);
		toolBar.add(toolPref);
		micStop = new ImageIcon(MagstripperGUI.class.getResource("icons/error_x.png"));
		micStart = new ImageIcon(MagstripperGUI.class.getResource("icons/microphone_next_move.png"));
		toolMic = new JButton(micStart);
		toolMic.setToolTipText("Start Mic Input");
		toolMic.setMargin(noborder);
		toolBar.add(toolMic);
		toolLock = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/key.png")));
		toolLock.setToolTipText("Test Lock");
		toolLock.setMargin(noborder);
		toolBar.add(toolLock);
		toolBar.addSeparator();
		toolConv = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/reload.png")));
		toolConv.setToolTipText("Conversion Tool");
		toolConv.setMargin(noborder);
		toolBar.add(toolConv);
		toolTable = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/calendar.png")));
		toolTable.setToolTipText("Character Table");
		toolTable.setMargin(noborder);
		toolBar.add(toolTable);
		toolLockScreen = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/lockscreen.png")));
		toolLockScreen.setToolTipText("Lock Screen");
		toolLockScreen.setMargin(noborder);
		toolBar.add(toolLockScreen);
		
		toolBar.addSeparator();
		toolHelp = new JButton(new ImageIcon(MagstripperGUI.class.getResource("icons/question_mark.png")));
		toolHelp.setToolTipText("Help");
		toolHelp.setMargin(noborder);
		//toolBar.add(toolHelp); //TODO create a help system and crap
		return toolBar;
	}
	
	/**
	 * Creates the bottom status bar
	 * @return the status bar
	 */
	private JPanel createStatusBar(){
		micStatusRed = new ImageIcon(MagstripperGUI.class.getResource("icons/circle_red16.png"));
		micStatusGreen = new ImageIcon(MagstripperGUI.class.getResource("icons/circle_green16.png"));
		JPanel statusBar = new JPanel(true);
		statusBar.setLayout(new BoxLayout(statusBar, BoxLayout.X_AXIS));
		micStatusLabel = new JLabel();
		micStatusLabel.setIcon(micStatusRed);
		micStatusLabel.setBorder(BorderFactory.createEtchedBorder());
		doorStatus = new JLabel(" Door: Locked    ");
		userCount = new JLabel(" 0 Users In Table  ");
		cardCount = new JLabel(" 0 Cards In Table  ");
		swipeCount = new JLabel(" 0 Swipes Recorded  ");
		doorStatus.setBorder(BorderFactory.createEtchedBorder());
		userCount.setBorder(BorderFactory.createEtchedBorder());
		cardCount.setBorder(BorderFactory.createEtchedBorder());
		swipeCount.setBorder(BorderFactory.createEtchedBorder());
		statusBar.add(micStatusLabel);
		statusBar.add(Box.createGlue());
		statusBar.add(swipeCount);
		statusBar.add(userCount);
		statusBar.add(cardCount);
		statusBar.add(doorStatus);
		return statusBar;
	}
	
	/**
	 * Creates the tabs and all the content within the tab's frame
	 * @return the tab pane
	 */
	private void createTabs(){
		tabs = new JTabbedPane();
		JPanel cardReader = new JPanel(new BorderLayout(), true);
		
		swipeListModel = new DefaultListModel();
		charListModel = new DefaultListModel();
		
		swipeList = new JList(swipeListModel);
		swipeList.setDoubleBuffered(true);
		charList = new JList(charListModel);
		charList.setDoubleBuffered(true);
		JScrollPane swipeScroll = new JScrollPane(swipeList);//west
		swipeScroll.setDoubleBuffered(true);
		JScrollPane charScroll = new JScrollPane(charList);
		charScroll.setDoubleBuffered(true);
		swipeScroll.setMinimumSize(new Dimension(150,80));
		charScroll.setMinimumSize(new Dimension(150,180));		
		swipeScroll.setPreferredSize(new Dimension(150,100));
		charScroll.setPreferredSize(new Dimension(150,200));
		swipeScroll.setToolTipText("New Swipes Will Show Up Here");
		charScroll.setToolTipText("Binary Is Broken Into Characters For The Current Swipe");
		swipeScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		charScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		JLabel swipeHead = new JLabel(" Swipes ");
		swipeHead.setBorder(BorderFactory.createEtchedBorder());
		JLabel charHead = new JLabel(" Decoded Chars ");
		charHead.setBorder(BorderFactory.createEtchedBorder());
		swipeScroll.setColumnHeaderView(swipeHead);
		charScroll.setColumnHeaderView(charHead);
		swipeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		charList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		swipeList.setLayoutOrientation(JList.VERTICAL);
		charList.setLayoutOrientation(JList.VERTICAL);
		JSplitPane westSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, swipeScroll, charScroll);
		westSplit.setResizeWeight(.2);
		westSplit.setPreferredSize(new Dimension(150,10));
		cardReader.add(westSplit, BorderLayout.WEST); //end west
		
		JPanel cardCenter = new JPanel(new BorderLayout(),true);
		JPanel cardCenterNorth = new JPanel(new BorderLayout(), true);
		JPanel cardCenterNorthCenter = new JPanel();
		cardCenterNorthCenter.setLayout(new BoxLayout(cardCenterNorthCenter, BoxLayout.Y_AXIS));
		ascii = new JTextPane();
		binary = new JTextPane();
		binary.setToolTipText("Decoded Characters Shown With Alternate Shading, CRC Bits Shown In Light Blue");
		rawbinary = new JTextPane();
		rawbinary.setToolTipText("Data That Is Decoded Shown In Bold, LRC Character Shown In Light Blue");
		
		ascii.setEditable(false);binary.setEditable(false);rawbinary.setEditable(false);
		Insets mainboxes = new Insets(0,3,0,3);
		ascii.setMargin(mainboxes);binary.setMargin(mainboxes);rawbinary.setMargin(mainboxes);
		JScrollPane asciiScroll = new JScrollPane(ascii);
		JScrollPane binaryScroll = new JScrollPane(binary);
		JScrollPane rawbinaryScroll = new JScrollPane(rawbinary);
		asciiScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		asciiScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		binaryScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		binaryScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		rawbinaryScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		rawbinaryScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		asciiScroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Decoded ASCII"));
		binaryScroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Binary Data"));
		rawbinaryScroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Raw Binary Data"));
		cardCenterNorthCenter.add(asciiScroll);
		cardCenterNorthCenter.add(binaryScroll);
		cardCenterNorthCenter.add(rawbinaryScroll);
		
		JPanel cardCenterNorthEast = new JPanel();//centernortheast stats
		cardCenterNorthEast.setLayout(new BoxLayout(cardCenterNorthEast, BoxLayout.Y_AXIS));
		characters = new JTextArea("0");
		lrc = new JTextArea("");
		crcerr = new JTextArea("0");
		lrcerr = new JTextArea("0");
		characters.setRows(1);lrc.setRows(1);crcerr.setRows(1);lrcerr.setRows(1);
		characters.setMargin(mainboxes);characters.setEditable(false);
		lrc.setMargin(mainboxes);lrc.setEditable(false);
		crcerr.setMargin(mainboxes);crcerr.setEditable(false);
		lrcerr.setMargin(mainboxes);lrcerr.setEditable(false);
		JScrollPane charactersScroll = new JScrollPane(characters);
		JScrollPane lrcScroll = new JScrollPane(lrc);
		JScrollPane crcerrScroll = new JScrollPane(crcerr);
		JScrollPane lrcerrScroll = new JScrollPane(lrcerr);
		charactersScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		charactersScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		lrcScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		lrcScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		lrcerrScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		lrcerrScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		crcerrScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		crcerrScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		charactersScroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Chars"));
		lrcScroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"LRC"));
		JPanel zeroPanel = new JPanel(new GridLayout(2,1));
		zeroPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Errors"));
		JPanel zeroTop = new JPanel();JPanel zeroBottom = new JPanel();
		zeroTop.setLayout(new BoxLayout(zeroTop, BoxLayout.X_AXIS));
		zeroBottom.setLayout(new BoxLayout(zeroBottom, BoxLayout.X_AXIS));
		zeroPanel.add(zeroTop);
		zeroPanel.add(zeroBottom);
		zeroTop.add(new JLabel("CRC "));
		zeroTop.add(Box.createRigidArea(new Dimension(2,0)));
		zeroTop.add(crcerrScroll);
		zeroBottom.add(new JLabel("LRC  "));
		zeroBottom.add(lrcerrScroll);
		cardCenterNorthEast.add(charactersScroll);
		cardCenterNorthEast.add(lrcScroll);
		cardCenterNorthEast.add(zeroPanel);
		cardCenterNorthEast.add(Box.createVerticalGlue());
		cardCenterNorth.add(cardCenterNorthEast, BorderLayout.EAST);
		cardCenterNorth.add(cardCenterNorthCenter, BorderLayout.CENTER);
		
		JPanel cardCenterNorthSouth = new JPanel(new GridLayout(1,4));//start stats grids centernorthsouth
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;gbc.weightx = 1;
		JPanel audioStats = new JPanel(new GridBagLayout());
		audioStats.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Audio Stats"));
		JLabel lchannels = new JLabel(" Channels: ");lchannels.setOpaque(true);
		JLabel lformat = new JLabel(" Format: ");lformat.setOpaque(true);
		JLabel lsampleRate = new JLabel(" Sample Rate: ");lsampleRate.setOpaque(true);
		JLabel lframeLength = new JLabel(" Frame Length: ");lframeLength.setOpaque(true);
		JLabel lbitsPerSample = new JLabel(" Bits Per Sample: ");lbitsPerSample.setOpaque(true);
		JLabel lbitOrder = new JLabel(" Bit Order: ");lbitOrder.setOpaque(true);
		tchannels = new JLabel("Mono ");tchannels.setOpaque(true);
		tformat = new JLabel("PCM_SIGNED ");tformat.setOpaque(true);
		tsampleRate = new JLabel("0 ");tsampleRate.setOpaque(true);
		tframeLength = new JLabel("0 ");tframeLength.setOpaque(true);
		tbitsPerSample = new JLabel("0 ");tbitsPerSample.setOpaque(true);
		tbitOrder = new JLabel("Little Endian ");tbitOrder.setOpaque(true);
		Color shaded = new Color(210,210,210);
		lchannels.setBackground(shaded);tchannels.setBackground(shaded);
		lsampleRate.setBackground(shaded);tsampleRate.setBackground(shaded);
		lbitsPerSample.setBackground(shaded);tbitsPerSample.setBackground(shaded);
		gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 2;audioStats.add(lchannels,gbc);
		gbc.gridx = 2;gbc.gridy = 0;gbc.gridwidth = 1;audioStats.add(tchannels,gbc);
		gbc.gridx = 0;gbc.gridy = 1;gbc.gridwidth = 2;audioStats.add(lformat,gbc);
		gbc.gridx = 2;gbc.gridy = 1;gbc.gridwidth = 1;audioStats.add(tformat,gbc);
		gbc.gridx = 0;gbc.gridy = 2;gbc.gridwidth = 2;audioStats.add(lsampleRate,gbc);
		gbc.gridx = 2;gbc.gridy = 2;gbc.gridwidth = 1;audioStats.add(tsampleRate,gbc);
		gbc.gridx = 0;gbc.gridy = 3;gbc.gridwidth = 2;audioStats.add(lframeLength,gbc);
		gbc.gridx = 2;gbc.gridy = 3;gbc.gridwidth = 1;audioStats.add(tframeLength,gbc);
		gbc.gridx = 0;gbc.gridy = 4;gbc.gridwidth = 2;audioStats.add(lbitsPerSample,gbc);
		gbc.gridx = 2;gbc.gridy = 4;gbc.gridwidth = 1;audioStats.add(tbitsPerSample,gbc);
		gbc.gridx = 0;gbc.gridy = 5;gbc.gridwidth = 2;audioStats.add(lbitOrder,gbc);
		gbc.gridx = 2;gbc.gridy = 5;gbc.gridwidth = 1;audioStats.add(tbitOrder,gbc);
		lchannels.setToolTipText("Audio Sample Channels, Mono or Stereo");tchannels.setToolTipText("Audio Sample Channels, Mono or Stereo");
		lformat.setToolTipText("Audio Sample Format, 16bit Is Signed 8bit is Unsigned");tformat.setToolTipText("Audio Sample Format, 16bit Is Signed 8bit is Unsigned");
		lsampleRate.setToolTipText("Sample Rate (Per Second) Of The Audio Sample");tsampleRate.setToolTipText("Sample Rate (Per Second) Of The Audio Sample");
		lframeLength.setToolTipText("Number Of Samples Contained In Each Audio Frame");tframeLength.setToolTipText("Number Of Samples Contained In Each Audio Frame");
		lbitsPerSample.setToolTipText("Number Of Bits Used To Store Each Sample");tbitsPerSample.setToolTipText("Number Of Bits Used To Store Each Sample");
		lbitOrder.setToolTipText("Bit Order Of Audio Sample, Should Always Be Little Endian");tbitOrder.setToolTipText("Bit Order Of Audio Sample, Should Always Be Little Endian");
		cardCenterNorthSouth.add(audioStats);
		
		JPanel trackStats = new JPanel(new GridBagLayout());
		trackStats.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Track Stats"));
		JLabel lzeroLevel = new JLabel(" Zero Level: ");lzeroLevel.setOpaque(true);
		JLabel lpositiveThreshold = new JLabel(" Positive Threshold: ");lpositiveThreshold.setOpaque(true);
		JLabel lnegativeThreshold = new JLabel(" Negative Threshold: ");lnegativeThreshold.setOpaque(true);
		JLabel lpositivePeaks = new JLabel(" Positive Peaks: ");lpositivePeaks.setOpaque(true);
		JLabel lnegativePeaks = new JLabel(" Negative Peaks: ");lnegativePeaks.setOpaque(true);
		JLabel ltotalPeaks = new JLabel(" Total Peaks: ");ltotalPeaks.setOpaque(true);
		tzeroLevel = new JLabel("0 ");tzeroLevel.setOpaque(true);
		tpositiveThreshold = new JLabel("0 ");tpositiveThreshold.setOpaque(true);
		tnegativeThreshold = new JLabel("0 ");tnegativeThreshold.setOpaque(true);
		tpositivePeaks = new JLabel("0 ");tpositivePeaks.setOpaque(true);
		tnegativePeaks = new JLabel("0");tnegativePeaks.setOpaque(true);
		ttotalPeaks = new JLabel("0 ");
		lzeroLevel.setBackground(shaded);tzeroLevel.setBackground(shaded);
		lnegativeThreshold.setBackground(shaded);tnegativeThreshold.setBackground(shaded);
		lpositivePeaks.setBackground(shaded);tpositivePeaks.setBackground(shaded);
		gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 2;trackStats.add(lzeroLevel,gbc);
		gbc.gridx = 2;gbc.gridy = 0;gbc.gridwidth = 1;trackStats.add(tzeroLevel,gbc);
		gbc.gridx = 0;gbc.gridy = 1;gbc.gridwidth = 2;trackStats.add(lpositiveThreshold,gbc);
		gbc.gridx = 2;gbc.gridy = 1;gbc.gridwidth = 1;trackStats.add(tpositiveThreshold,gbc);
		gbc.gridx = 0;gbc.gridy = 2;gbc.gridwidth = 2;trackStats.add(lnegativeThreshold,gbc);
		gbc.gridx = 2;gbc.gridy = 2;gbc.gridwidth = 1;trackStats.add(tnegativeThreshold,gbc);
		gbc.gridx = 0;gbc.gridy = 3;gbc.gridwidth = 2;trackStats.add(ltotalPeaks,gbc);
		gbc.gridx = 2;gbc.gridy = 3;gbc.gridwidth = 1;trackStats.add(ttotalPeaks,gbc);
		gbc.gridx = 0;gbc.gridy = 4;gbc.gridwidth = 2;trackStats.add(lpositivePeaks,gbc);
		gbc.gridx = 2;gbc.gridy = 4;gbc.gridwidth = 1;trackStats.add(tpositivePeaks,gbc);
		gbc.gridx = 0;gbc.gridy = 5;gbc.gridwidth = 2;trackStats.add(lnegativePeaks,gbc);
		gbc.gridx = 2;gbc.gridy = 5;gbc.gridwidth = 1;trackStats.add(tnegativePeaks,gbc);
		lzeroLevel.setToolTipText("The Adjusted Zero Point Used For This Swipe");tzeroLevel.setToolTipText("The Adjusted Zero Point Used For This Swipe");
		lpositiveThreshold.setToolTipText("The Upper Threshold Point Used As A Noise Level To Detect High Peaks");tpositiveThreshold.setToolTipText("The Upper Threshold Point Used As A Noise Level To Detect High Peaks");
		lnegativeThreshold.setToolTipText("The Lower Threshold Point Used As A Noise Level To Detect Low Peaks");tnegativeThreshold.setToolTipText("The Lower Threshold Point Used As A Noise Level To Detect Low Peaks");
		lpositivePeaks.setToolTipText("Number Of High Peaks Detected");tpositivePeaks.setToolTipText("Number Of High Peaks Detected");
		lnegativePeaks.setToolTipText("Number Of Low Peaks Detected");tnegativePeaks.setToolTipText("Number Of Low Peaks Detected");
		ltotalPeaks.setToolTipText("Number Of Total Peaks Detected");ttotalPeaks.setToolTipText("Number Of Total Peaks Detected");
		cardCenterNorthSouth.add(trackStats);
		
		JPanel decodeStats = new JPanel(new GridBagLayout());
		decodeStats.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Decode Stats"));
		JLabel lleading = new JLabel(" Leading Zeros: ");lleading.setOpaque(true);
		JLabel ltrailing = new JLabel(" Trailing Zeros: ");ltrailing.setOpaque(true);
		JLabel lbitsPerChar = new JLabel(" Bits Per Char: ");lbitsPerChar.setOpaque(true);
		JLabel lswipeReverse = new JLabel(" Swiped Backwards: ");lswipeReverse.setOpaque(true);
		JLabel lleadingOne = new JLabel(" Leading One: ");lleadingOne.setOpaque(true);
		JLabel lrawLength = new JLabel(" Raw Length");lrawLength.setOpaque(true);
		tleading = new JLabel("0 ");tleading.setOpaque(true);
		ttrailing = new JLabel("0 ");ttrailing.setOpaque(true);
		tbitsPerChar = new JLabel("0 ");tbitsPerChar.setOpaque(true);
		tswipeReverse = new JLabel("No ");tswipeReverse.setOpaque(true);
		tleadingOne = new JLabel("No ");tleadingOne.setOpaque(true);
		trawLength = new JLabel("0 ");trawLength.setOpaque(true);
		lleading.setBackground(shaded);tleading.setBackground(shaded);
		lbitsPerChar.setBackground(shaded);tbitsPerChar.setBackground(shaded);
		lleadingOne.setBackground(shaded);tleadingOne.setBackground(shaded);
		gbc.gridx = 0;gbc.gridy = 0;gbc.gridwidth = 2;decodeStats.add(lleading,gbc);
		gbc.gridx = 2;gbc.gridy = 0;gbc.gridwidth = 1;decodeStats.add(tleading,gbc);
		gbc.gridx = 0;gbc.gridy = 1;gbc.gridwidth = 2;decodeStats.add(ltrailing,gbc);
		gbc.gridx = 2;gbc.gridy = 1;gbc.gridwidth = 1;decodeStats.add(ttrailing,gbc);
		gbc.gridx = 0;gbc.gridy = 2;gbc.gridwidth = 2;decodeStats.add(lbitsPerChar,gbc);
		gbc.gridx = 2;gbc.gridy = 2;gbc.gridwidth = 1;decodeStats.add(tbitsPerChar,gbc);
		gbc.gridx = 0;gbc.gridy = 3;gbc.gridwidth = 2;decodeStats.add(lswipeReverse,gbc);
		gbc.gridx = 2;gbc.gridy = 3;gbc.gridwidth = 1;decodeStats.add(tswipeReverse,gbc);
		gbc.gridx = 0;gbc.gridy = 4;gbc.gridwidth = 2;decodeStats.add(lleadingOne,gbc);
		gbc.gridx = 2;gbc.gridy = 4;gbc.gridwidth = 1;decodeStats.add(tleadingOne,gbc);
		gbc.gridx = 0;gbc.gridy = 5;gbc.gridwidth = 2;decodeStats.add(lrawLength,gbc);
		gbc.gridx = 2;gbc.gridy = 5;gbc.gridwidth = 1;decodeStats.add(trawLength,gbc);
		lleading.setToolTipText("Number Of Zero Bits Before Start Sentinel");tleading.setToolTipText("Number Of Zero Bits Before Start Sentinel");
		ltrailing.setToolTipText("Number Of Zero Bits After The LRC Character");ttrailing.setToolTipText("Number Of Zero Bits After The LRC Character");
		lbitsPerChar.setToolTipText("Number Of Bits Used For Each Decoded Character");tbitsPerChar.setToolTipText("Number Of Bits Used For Each Decoded Character");
		lswipeReverse.setToolTipText("Displays If This Swipe Was Read The Wrong Way");tswipeReverse.setToolTipText("Displays If This Swipe Was Read The Wrong Way");
		lleadingOne.setToolTipText("Displays If There Was A Non Standard Leading One Bit");tleadingOne.setToolTipText("Displays If There Was A Non Standard Leading One Bit");
		lrawLength.setToolTipText("The Number Of Bits Read");trawLength.setToolTipText("The Number Of Bits Read");
		cardCenterNorthSouth.add(decodeStats);
		cardCenterNorth.add(cardCenterNorthSouth, BorderLayout.SOUTH);
		cardCenter.add(cardCenterNorth, BorderLayout.NORTH);
		
		JPanel consolePanel = new JPanel(new BorderLayout(),true); //console panel cardcentercenter
		console = new JTextPane();
		console.setMargin(mainboxes);console.setEditable(false);
		JScrollPane consoleScroll = new JScrollPane(console);
		consoleScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		consoleScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		JPanel consoleLogTop = new JPanel();
		consoleLogTop.setLayout(new BoxLayout(consoleLogTop,BoxLayout.X_AXIS));
		consoleLogTop.setBorder(BorderFactory.createEtchedBorder());
		String[] consoleLogComboText = { " Error", " Warning", " Info", " Debug" };
		consoleLogCombo = new JComboBox(consoleLogComboText);
		consoleLogCombo.setMaximumSize(new Dimension(80,21));
		consoleLogCombo.setPreferredSize(new Dimension(80,21));
		consoleLogCombo.setSelectedIndex(1);
		consoleLogCombo.setToolTipText("Filter Out Log Entries Lower Than This Value");
		consoleLogTop.add(Box.createRigidArea(new Dimension(4,0)));
		JLabel cfilter = new JLabel("Filter");
		consoleLogTop.add(cfilter);
		consoleLogTop.add(Box.createRigidArea(new Dimension(3,0)));
		consoleLogTop.add(consoleLogCombo);
		JLabel cdebug = new JLabel("Debug");
		Dimension maxsize = new Dimension(50,50);
		Dimension minsize = new Dimension(50,0);
		cdebugNum = new JTextField("0",4);
		cdebugNum.setMaximumSize(maxsize);
		cdebugNum.setMinimumSize(minsize);
		cdebugNum.setEditable(false);
		cdebugNum.setToolTipText("The Number of Debug Level Messages Logged");
		cdebugNum.setBorder(BorderFactory.createLoweredBevelBorder());
		JLabel cinfo = new JLabel("Info");
		cinfoNum = new JTextField("0",4);
		cinfoNum.setMaximumSize(maxsize);
		cinfoNum.setMinimumSize(minsize);
		cinfoNum.setEditable(false);
		cinfoNum.setToolTipText("The Number of Info Level Messages Logged");
		cinfoNum.setBorder(BorderFactory.createLoweredBevelBorder());
		JLabel cwarn = new JLabel("Warning");
		cwarnNum = new JTextField("0",4);
		cwarnNum.setMaximumSize(maxsize);
		cwarnNum.setMinimumSize(minsize);
		cwarnNum.setEditable(false);
		cwarnNum.setToolTipText("The Number of Warning Level Messages Logged");
		cwarnNum.setBorder(BorderFactory.createLoweredBevelBorder());
		JLabel cerror = new JLabel("Error");
		cerrorNum = new JTextField("0",4);
		cerrorNum.setMaximumSize(maxsize);
		cerrorNum.setMinimumSize(minsize);
		cerrorNum.setEditable(false);
		cerrorNum.setToolTipText("The Number of Error Level Messages Logged");
		cerrorNum.setBorder(BorderFactory.createLoweredBevelBorder());
		consoleLogTop.add(Box.createRigidArea(new Dimension(10,0)));consoleLogTop.add(cdebug);consoleLogTop.add(Box.createRigidArea(new Dimension(4,0)));consoleLogTop.add(cdebugNum);
		consoleLogTop.add(Box.createRigidArea(new Dimension(8,0)));consoleLogTop.add(cinfo);consoleLogTop.add(Box.createRigidArea(new Dimension(4,0)));consoleLogTop.add(cinfoNum);
		consoleLogTop.add(Box.createRigidArea(new Dimension(8,0)));consoleLogTop.add(cwarn);consoleLogTop.add(Box.createRigidArea(new Dimension(4,0)));consoleLogTop.add(cwarnNum);
		consoleLogTop.add(Box.createRigidArea(new Dimension(8,0)));consoleLogTop.add(cerror);consoleLogTop.add(Box.createRigidArea(new Dimension(4,0)));consoleLogTop.add(cerrorNum);
		consoleLogTop.add(Box.createHorizontalGlue());
		consolePanel.add(consoleLogTop, BorderLayout.NORTH);
		consolePanel.add(consoleScroll, BorderLayout.CENTER);
		cardCenter.add(consolePanel, BorderLayout.CENTER);//end console panel (cardcentercenter)
		cardReader.add(cardCenter, BorderLayout.CENTER);
		
		JPanel userTable = new JPanel(new BorderLayout(), true); //usertable tab
		JPanel userCenter = new JPanel(new BorderLayout(), true); //center panel, buttons, and 2 tables (in a split)
		JPanel userCenterTop = new JPanel();//buttons pane
		userCenterTop.setLayout(new BoxLayout(userCenterTop, BoxLayout.X_AXIS));
		userCenterTop.setBorder(BorderFactory.createEtchedBorder());
		userAdd = new JButton("Add User",new ImageIcon(MagstripperGUI.class.getResource("icons/address_next_move_add_insert.png")));
		userAdd.setToolTipText("Add A User To The Table");
		userDel = new JButton("Delete User",new ImageIcon(MagstripperGUI.class.getResource("icons/addresss_delete.png")));
		userDel.setToolTipText("Delete A User From The Table");
		userOpen = new JButton("Open Table", new ImageIcon(MagstripperGUI.class.getResource("icons/folder_document.png")));
		userOpen.setToolTipText("Open A Saved Table From Disk");
		userSave = new JButton("Save Table", new ImageIcon(MagstripperGUI.class.getResource("icons/document_floppy_save.png")));
		userSave.setToolTipText("Save The Table To Disk");
		userClear = new JButton("Clear Table", new ImageIcon(MagstripperGUI.class.getResource("icons/clear_wipe_erase.png")));
		userClear.setToolTipText("Clear All Table Data");
		userAdd.setMargin(new Insets(0,4,0,3));userDel.setMargin(new Insets(0,4,0,3));
		userOpen.setMargin(new Insets(0,4,0,3));userSave.setMargin(new Insets(0,4,0,3));userClear.setMargin(new Insets(0,4,0,3));
		userCenterTop.add(userAdd);userCenterTop.add(Box.createRigidArea(new Dimension(8,8)));userCenterTop.add(userDel);userCenterTop.add(Box.createRigidArea(new Dimension(8,8)));
		userCenterTop.add(userOpen);userCenterTop.add(Box.createRigidArea(new Dimension(8,8)));userCenterTop.add(userSave);userCenterTop.add(Box.createRigidArea(new Dimension(8,8)));
		userCenterTop.add(userClear);
		userCenter.add(userCenterTop,BorderLayout.NORTH); //end top buttons
		
		JSplitPane userCenterSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);
		JSplitPane userLeft = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);
		userCenterSplit.setResizeWeight(.7);
		table = new JTable(mmodel.getTableModel());
		table.setOpaque(true);table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane tableScroll = new JScrollPane(table);
		table.getColumnModel().getColumn(0).setPreferredWidth(20);
		table.getColumnModel().getColumn(1).setPreferredWidth(40);
		table.getColumnModel().getColumn(2).setPreferredWidth(200);
		table.getColumnModel().getColumn(3).setPreferredWidth(200);
		table.getColumnModel().getColumn(4).setPreferredWidth(70);
		table.getColumnModel().getColumn(5).setPreferredWidth(80);
		table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
		userCenterSplit.add(tableScroll);
		
		JPanel tcardCenter = new JPanel(new BorderLayout(), true);
		JPanel tcardCenterTop = new JPanel();
		tcardCenterTop.setLayout(new BoxLayout(tcardCenterTop, BoxLayout.X_AXIS));
		tcardCenterTop.setBorder(BorderFactory.createEtchedBorder());
		cardAdd = new JButton("Add Card", new ImageIcon(MagstripperGUI.class.getResource("icons/human_keys_privacy.png")));
		cardDel = new JButton("Delete Card", new ImageIcon(MagstripperGUI.class.getResource("icons/delete_card.png")));
		cardAdd.setMargin(new Insets(0,4,0,3));cardDel.setMargin(new Insets(0,4,0,3));
		tcardCenterTop.add(cardAdd);tcardCenterTop.add(Box.createRigidArea(new Dimension(8,8)));tcardCenterTop.add(cardDel);
		tcardCenter.add(tcardCenterTop, BorderLayout.NORTH);//end buttons
		cardTable = new JTable(mmodel.getCardTableModel());
		cardTable.setOpaque(true);cardTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane cardScroll = new JScrollPane(cardTable);
		cardTable.getColumnModel().getColumn(0).setPreferredWidth(30);
		cardTable.getColumnModel().getColumn(1).setPreferredWidth(45);
		cardTable.getColumnModel().getColumn(2).setPreferredWidth(300);
		cardTable.getColumnModel().getColumn(3).setPreferredWidth(80);
		cardTable.getColumnModel().getColumn(4).setPreferredWidth(80);
		cardTable.getColumnModel().getColumn(5).setPreferredWidth(80);
		cardTable.getColumnModel().getColumn(6).setPreferredWidth(50);
		String[] opts = {"Both", "Forward", "Backward"};
		cardTable.getColumnModel().getColumn(3).setCellEditor(new DefaultCellEditor(new JComboBox(opts)));
		tcardCenter.add(cardScroll, BorderLayout.CENTER);
		userCenterSplit.add(tcardCenter);
		userCenter.add(userCenterSplit, BorderLayout.CENTER);
		userTable.add(userLeft, BorderLayout.WEST);
		userTable.add(userCenter, BorderLayout.CENTER);
		
		JPanel systemLog = new JPanel(new BorderLayout(), true); //system log tab
		JPanel systemLogTop = new JPanel();
		systemLogTop.setLayout(new BoxLayout(systemLogTop,BoxLayout.X_AXIS));
		String[] systemLogComboText = { " Error", " Warning", " Info", " Debug" };
		systemLogCombo = new JComboBox(systemLogComboText);
		systemLogCombo.setMaximumSize(new Dimension(80,21));
		systemLogCombo.setPreferredSize(new Dimension(80,21));
		systemLogCombo.setSelectedIndex(2);
		systemLogCombo.setToolTipText("Filter Out Log Entries Lower Than This Value");
		systemLogTop.add(Box.createRigidArea(new Dimension(4,0)));
		JLabel filter = new JLabel("Filter");
		systemLogTop.add(filter);
		systemLogTop.add(Box.createRigidArea(new Dimension(3,0)));
		systemLogTop.add(systemLogCombo);
		
		JLabel debug = new JLabel("Debug");
		debugNum = new JTextField("0",4);
		debugNum.setMaximumSize(maxsize);
		debugNum.setMinimumSize(minsize);
		debugNum.setEditable(false);
		debugNum.setToolTipText("The Number of Debug Level Messages Logged");
		debugNum.setBorder(BorderFactory.createLoweredBevelBorder());
		JLabel info = new JLabel("Info");
		infoNum = new JTextField("0",4);
		infoNum.setMaximumSize(maxsize);
		infoNum.setMinimumSize(minsize);
		infoNum.setEditable(false);
		infoNum.setToolTipText("The Number of Info Level Messages Logged");
		infoNum.setBorder(BorderFactory.createLoweredBevelBorder());
		JLabel warn = new JLabel("Warning");
		warnNum = new JTextField("0",4);
		warnNum.setMaximumSize(maxsize);
		warnNum.setMinimumSize(minsize);
		warnNum.setEditable(false);
		warnNum.setToolTipText("The Number of Warning Level Messages Logged");
		warnNum.setBorder(BorderFactory.createLoweredBevelBorder());
		JLabel error = new JLabel("Error");
		errorNum = new JTextField("0",4);
		errorNum.setMaximumSize(maxsize);
		errorNum.setMinimumSize(minsize);
		errorNum.setEditable(false);
		errorNum.setToolTipText("The Number of Error Level Messages Logged");
		errorNum.setBorder(BorderFactory.createLoweredBevelBorder());
		systemLogTop.add(Box.createRigidArea(new Dimension(10,0)));systemLogTop.add(debug);systemLogTop.add(Box.createRigidArea(new Dimension(4,0)));systemLogTop.add(debugNum);
		systemLogTop.add(Box.createRigidArea(new Dimension(8,0)));systemLogTop.add(info);systemLogTop.add(Box.createRigidArea(new Dimension(4,0)));systemLogTop.add(infoNum);
		systemLogTop.add(Box.createRigidArea(new Dimension(8,0)));systemLogTop.add(warn);systemLogTop.add(Box.createRigidArea(new Dimension(4,0)));systemLogTop.add(warnNum);
		systemLogTop.add(Box.createRigidArea(new Dimension(8,0)));systemLogTop.add(error);systemLogTop.add(Box.createRigidArea(new Dimension(4,0)));systemLogTop.add(errorNum);
		systemLogTop.add(Box.createHorizontalGlue());
		systemLog.add(systemLogTop, BorderLayout.NORTH);
		systemText = new JTextPane();
		systemText.setEditable(false);
		systemText.setMargin(mainboxes);
		JScrollPane systemScroll = new JScrollPane(systemText);
		systemLog.add(systemScroll,BorderLayout.CENTER);
		systemScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS); //end system log tab
		updateSystemConsole();
		//add the tabs to the bar
		tabs.addTab("Card Reader", cardReader);
		tabs.addTab("User Table", userTable);
		tabs.addTab("System Log", systemLog);
	}
	
	/**
	 * Displays the about screen in a dialog
	 */
	public void displayAbout(){
	    Point frameLoc = this.getLocationOnScreen();
	    Dimension sourceSize = this.getSize();
	    
		JDialog.setDefaultLookAndFeelDecorated(true); //make it pretty
	    JDialog about = new JDialog(this,"About",true);
		JPanel aboutPanel = new JPanel(new BorderLayout());
		aboutPanel.setBorder(BorderFactory.createEtchedBorder());
		JLabel aboutTitle = new JLabel("  Magstripper "+mmodel.getVersion()+"  ");
		aboutTitle.setHorizontalAlignment(JLabel.CENTER);
		aboutTitle.setBorder(BorderFactory.createRaisedBevelBorder());
		
		JTextPane aboutCenter = new JTextPane();
		aboutCenter.setEditable(false);
		aboutCenter.setMargin(new Insets(5,5,5,5));
		StyledDocument doc = aboutCenter.getStyledDocument();
		Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
		
		String[] popContent = {"Thanks to sephail.net and stripesnoop.sf.net for\nproof of concept and for peaking my interest in\nmagnetic stripes.\n\n","Icons created by Lim Chee Aun from phoenity.com\nand released under the Creative Commons\nAttribution-NonCommercial 2.5 License\n\n","Magstripper Copyright 2006, 2007\nMagstripper comes with ABSOLUTELY NO WARRANTY;\nThis is free software, and you are welcome to\nredistributeit under certain conditions;\nfor details read gpl-3.0.txt"};
		try{
			doc.insertString(doc.getLength(),popContent[0],def);
			doc.insertString(doc.getLength(),popContent[1],def);
			doc.insertString(doc.getLength(),popContent[2],def);
		}catch(BadLocationException ble){
			mmodel.getLog().addMsg("Error: Could Not Render About Window, BadLocationException When Inserting Strings");
			JDialog.setDefaultLookAndFeelDecorated(true);
			JOptionPane.showMessageDialog(this, "Could Not Display About Box Properly", "About Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
			
		aboutPanel.add(aboutTitle, BorderLayout.NORTH);
		aboutPanel.add(aboutCenter, BorderLayout.CENTER);
		about.add(aboutPanel);
		about.pack();
		about.setLocation(sourceSize.width/2+frameLoc.x-about.getWidth()/2,sourceSize.height/2+frameLoc.y-about.getHeight()/2);
		about.setVisible(true);
	}
	
	/**
	 * Creates the GUI elements, and sets the default close operation
	 */
	public void createGUI() {
        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE); //we manually handle the close events
        this.setJMenuBar(createMenu());
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.setOpaque(true);
        this.add(createToolBar(), BorderLayout.NORTH);
        this.add(createStatusBar(), BorderLayout.SOUTH);
        createTabs();
        this.add(tabs,BorderLayout.CENTER);
        
        //Display the window.
        Dimension res = Toolkit.getDefaultToolkit().getScreenSize();
        this.setSize(950,700);
        if(res.getHeight() < 750 || res.getWidth() < 1000)
        	this.setSize( (int)(res.getWidth()-50), (int)(res.getHeight()-50));
        this.setLocationRelativeTo(null);//centers the window
    }
	
	/**
	 * Sets the GUI to visible
	 */
	public void showGUI(){
		this.setVisible(true);
	}
	
	/**
	 * Adds the mic listener
	 * @param a
	 */
	public void addMicListener(ActionListener a){ toolMic.addActionListener(a); micPref.addActionListener(a);	}
	/**
	 * Adds the about listener
	 * @param a
	 */
	public void addAboutListener(ActionListener a){ helpAbout.addActionListener(a); }
	/**
	 * Adds the Help listener
	 * @param a
	 */
	public void addHelpListener(ActionListener a){ helpHelp.addActionListener(a); toolHelp.addActionListener(a); }
	/**
	 * Adds the preference listener
	 * @param a
	 */
	public void addPrefListener(ActionListener a){ optionPref.addActionListener(a); toolPref.addActionListener(a); }
	/**
	 * Adds the exit listener
	 * @param a
	 */
	public void addExitListener(ActionListener a){ fileExit.addActionListener(a); }
	/**
	 * Adds the save listener
	 * @param a
	 */
	public void addSaveListener(ActionListener a){ fileSave.addActionListener(a); toolSave.addActionListener(a); }
	/**
	 * Adds the open listener
	 * @param a
	 */
	public void addOpenListener(ActionListener a){ fileOpen.addActionListener(a); toolOpen.addActionListener(a); }
	/**
	 * Adds the lock listener
	 * @param a
	 */
	public void addLockListener(ActionListener a){ toolLock.addActionListener(a); lockPref.addActionListener(a); }
	/**
	 * Adds the swipe list selection listener
	 * @param a
	 */
	
	public void addConversionToolListener(ActionListener a){ optionConv.addActionListener(a); toolConv.addActionListener(a); }
	
	public void addCharacterTableListener(ActionListener a){ optionTable.addActionListener(a); toolTable.addActionListener(a); }
	
	public void addLockScreenListener(ActionListener a){ optionLock.addActionListener(a); toolLockScreen.addActionListener(a); }
	
	public void addSwipeListSelectionListener(ListSelectionListener lsl){ swipeList.addListSelectionListener(lsl); }
	/**
	 * Adds the character list selection listener
	 * @param a
	 */
	public void addCharListSelectionListener(ListSelectionListener lsl){ charList.addListSelectionListener(lsl); }
	/**
	 * Adds the swipe console log combo box listener
	 * @param a
	 */
	public void addConsoleLogComboBoxListener(ActionListener a){ consoleLogCombo.addActionListener(a); }
	/**
	 * Adds the system log combo box listener
	 * @param a
	 */
	public void addSystemLogComboBoxListener(ActionListener a){ systemLogCombo.addActionListener(a); }
	/**
	 * Adds the user add listener
	 * @param a
	 */
	public void addUserAddListener(ActionListener a){ userAdd.addActionListener(a); tableAdd.addActionListener(a); }
	/**
	 * Adds the user delete listener
	 * @param a
	 */
	public void addUserDelListener(ActionListener a){ userDel.addActionListener(a); tableDel.addActionListener(a); }
	/**
	 * Adds the user open listener
	 * @param a
	 */
	public void addUserOpenListener(ActionListener a){ userOpen.addActionListener(a); tableOpen.addActionListener(a); }
	/**
	 * Adds the user save listener
	 * @param a
	 */
	public void addUserSaveListener(ActionListener a){ userSave.addActionListener(a); tableSave.addActionListener(a); }
	/**
	 * Adds the user clear listener
	 * @param a
	 */
	public void addUserClearListener(ActionListener a){ userClear.addActionListener(a); tableClear.addActionListener(a); }
	/**
	 * Adds the user table listener
	 * @param a
	 */
	public void addUserTableListener(ListSelectionListener lsl){ table.getSelectionModel().addListSelectionListener(lsl); }
	/**
	 * Adds the card add listener
	 * @param a
	 */
	public void addCardAddListener(ActionListener a){ cardAdd.addActionListener(a); }
	/**
	 * Adds the card delete listener
	 * @param a
	 */
	public void addCardDelListener(ActionListener a){ cardDel.addActionListener(a); }
	/**
	 * Switches tabs when the table menu items are selected
	 * @param a
	 */
	public void addTableTabListener(ActionListener a){
		fileOpen.addActionListener(a); toolOpen.addActionListener(a); fileSave.addActionListener(a); toolSave.addActionListener(a); toolSystem.addActionListener(a);
		tableAdd.addActionListener(a);tableDel.addActionListener(a);tableOpen.addActionListener(a);tableSave.addActionListener(a); toolSwipe.addActionListener(a);
		tableClear.addActionListener(a); fileSaveSystem.addActionListener(a); fileSaveConsole.addActionListener(a);
	}
	/**
	 * Adds the save swipe log listener
	 * @param a
	 */
	public void addSaveConsoleLogListener(ActionListener a){ fileSaveConsole.addActionListener(a); toolSwipe.addActionListener(a); }
	/**
	 * Adds the save system log listener
	 * @param a
	 */
	public void addSaveSystemLogListener(ActionListener a){ fileSaveSystem.addActionListener(a); toolSystem.addActionListener(a); }
	/**
	 * Adds the user table tooltip listener
	 * @param a
	 */
	public void addUserTableTooltipListener(MouseMotionListener mml){ table.getTableHeader().addMouseMotionListener(mml); }
	
	public void addUserTablePopupListener(MouseListener ml){ table.addMouseListener(ml); }
	
	/**
	 * Sets the listening state of the microphone button
	 * @param b true if the mic is listening, false if paused
	 */
	public void setListening(boolean b){
		if(!b){
			micStatusLabel.setIcon(micStatusRed);
			toolMic.setToolTipText("Start Mic Input");
			toolMic.setIcon(micStart);
			micPref.setIcon(micPrefStart);
		}else{
			micStatusLabel.setIcon(micStatusGreen);
			toolMic.setIcon(micStop);
			micPref.setIcon(micPrefStop);
			toolMic.setToolTipText("Stop Mic Input");
		}
	}
	
	/**
	 * Sets the locked state
	 * @param b true if locked false if unlocked
	 */
	public void setLocked(boolean b){
		if(b){
			doorStatus.setText(" Door: Locked    ");
		}else{
			doorStatus.setText(" Door: Unlocked ");
		}
	}
	
	/**
	 * Adds a swipe to the GUI
	 * @param s the swipe to add
	 */
	public void addSwipe(Swipe s){
		swipeListModel.insertElementAt(s.getTimestamp(),0);
		swipeList.setSelectedIndex(0);
		swipeList.ensureIndexIsVisible(0);
	}
	
	/**
	 * Updates the rest of the GUI based on the swipe selection
	 * @param s the swipe
	 */
	public void updateSwipeSelection(Swipe s){
		String dec = s.getDecodedString();
		if(mmodel.getSpaceChar() != ' '){ dec = dec.replace(' ', mmodel.getSpaceChar()); }
		ascii.setText(dec); //update the top section
		updateCharSelection(s);
		
		characters.setText(s.getDecodedString().length()+"");
		lrc.setText(s.getLRC());
		lrcerr.setText(s.getLRCErrors().size()+"");
		crcerr.setText(s.getCRCErrors().size()+"");
		
		tchannels.setText(s.getChannels()+" "); //update the audio stats
		tformat.setText(s.getFormat()+" ");
		tsampleRate.setText(s.getSampleRate()+" ");
		tframeLength.setText(s.getFrameLength()+" ");
		tbitsPerSample.setText(s.getBitsPerSample()+" ");
		tbitOrder.setText(s.getBitOrder()+" ");
	
		tzeroLevel.setText(s.getZeroLevel()+" "); //update the track stats
		tpositiveThreshold.setText(s.getPosThres()+" ");
		tnegativeThreshold.setText(s.getNegThres()+" ");
		ttotalPeaks.setText(s.getTotalNegPeaks() + s.getTotalPosPeaks()+" ");
		tpositivePeaks.setText(s.getTotalPosPeaks()+" ");
		tnegativePeaks.setText(s.getTotalNegPeaks()+" ");
		
		tleading.setText(s.getLeadingZeros()+" "); //update the decodestats
		ttrailing.setText(s.getTrailingZeros()+" ");
		tbitsPerChar.setText(s.getBitsPerChar()+" ");
		tswipeReverse.setText(s.getSwipedReverse()+" ");
		tleadingOne.setText(s.getLeadingOne()+" ");
		trawLength.setText(s.getRawLength()+" ");
		
		this.updateSwipeConsole(s);
		this.updateCharList(s);
	}
	
	/**
	 * Update the character selection based on which character is selected
	 * @param s the current swipe selected
	 */
	public void updateCharSelection(Swipe s){
		int selected = charList.getSelectedIndex();
		String strippedbin = s.getStrippedBinary();
		short strippedbinlen = s.getStrippedLength();
		int bpc = s.getBitsPerChar();
		if(bpc > 0){
			String[] binaryst = new String[strippedbinlen*2/bpc];
			for(int i=0;i<strippedbinlen/bpc;i++){
				binaryst[i*2] = strippedbin.substring(i*bpc,i*bpc+bpc-1);
				binaryst[i*2+1] = strippedbin.substring(i*bpc+bpc-1,i*bpc+bpc);
			}
			String[] binarystyle = new String[strippedbinlen*2/bpc];
			for(int i=0;i<strippedbinlen*2/bpc;i++){
				if(selected*2 != i && selected*2+1 != i){ //if this is not the selected char
					if(i%4 == 0){  binarystyle[i] = "normal";//non crc, white background
					}else if(i%4 == 1){  binarystyle[i] = "lightnormal";//crc, white background
					}else if(i%4 == 2){  binarystyle[i] = "greyback";//non crc grey background
					}else{  binarystyle[i] = "lightgreyback";//crc grey background
					}
				}else{ //it is the selected char
					if(i%4 == 0){  binarystyle[i] = "boldnormal";//non crc, white background
					}else if(i%4 == 1){  binarystyle[i] = "boldlightnormal";//crc, white background
					}else if(i%4 == 2){  binarystyle[i] = "boldgreyback";//non crc, grey background
					}else{	binarystyle[i] = "boldlightgreyback";//crc grey background
					}
				}		
			}
			StyledDocument sd = binary.getStyledDocument();
			addStyles(sd);
			for(int i=0;i<binaryst.length;i++){
				try {
					sd.insertString(sd.getLength(), binaryst[i], sd.getStyle(binarystyle[i]));
				} catch (BadLocationException e) {
					mmodel.getLog().addMsg("Error: BadLocationException Caught When Trying To Update Binary Data Selection");
					return;
				}
			}
			
			//new raw binary
			String rawbinstring = s.getRawBinary();
			short leadingzeros = s.getLeadingZeros();
			short trailingzeros = s.getTrailingZeros();
			int numchars = s.getDecodedString().length();
			String[] rawbinst = new String[leadingzeros + trailingzeros + numchars + 1]; //one for each leading/trailing, each char and lrc
			String[] rawbinstyle = new String[rawbinst.length];
			
			//leading zeros
			for(int i=0;i<leadingzeros;i++){
				rawbinst[i] = rawbinstring.substring(i,i+1);
				if(i%2==0)//work around for paragraph wrapping
					rawbinstyle[i] = "normal";
				else
					rawbinstyle[i] = "normal1";
			}
			//chars
			for(int i=leadingzeros;i<leadingzeros+numchars;i++){
				rawbinst[i] = rawbinstring.substring( ((i-leadingzeros)*s.getBitsPerChar())+leadingzeros , ((i-leadingzeros)*s.getBitsPerChar())+leadingzeros+s.getBitsPerChar());
				if(selected == (i-leadingzeros)){
					rawbinstyle[i] = "boldgreyback";
				}else{
					if(i%2==0)//work around for paragraph wrapping
						rawbinstyle[i] = "boldnormal";
					else
						rawbinstyle[i] = "boldnormal1";
				}
			}
			//lrc
			if(s.getLRC() != null){
				rawbinst[leadingzeros+numchars] = rawbinstring.substring(
						leadingzeros+numchars*s.getBitsPerChar(),leadingzeros+s.getBitsPerChar()*(numchars+1));
			}else{
				rawbinst[leadingzeros+numchars] = "";
			}
			//trailing
			rawbinstyle[leadingzeros+numchars] = "boldlightnormal";
			for(int i=leadingzeros+numchars+1;i<leadingzeros+numchars+1+trailingzeros;i++){
				rawbinst[i] = rawbinstring.substring((i-(numchars+1))+s.getBitsPerChar()*(numchars+1), 
						(i-(numchars+1))+s.getBitsPerChar()*(numchars+1)+1);
				if(i%2==0) //work around for paragraph wrapping
					rawbinstyle[i] = "normal";
				else
					rawbinstyle[i] = "normal1";
			}
			
			StyledDocument sdraw = rawbinary.getStyledDocument();
			addStyles(sdraw);
			for(int i=0;i<rawbinst.length;i++){
				try{
					sdraw.insertString(sdraw.getLength(), rawbinst[i], sdraw.getStyle(rawbinstyle[i]));
				}catch (BadLocationException e){
					mmodel.getLog().addMsg("Error: BadLocationException Caught When Trying To Update Raw Binary Selection");
					return;
				}
			}
			
		}else{//couldnt determine bpc, we're boned
			StyledDocument sd = binary.getStyledDocument();
			try {
				sd.remove(0, sd.getLength());
				sd.insertString(sd.getLength(), s.getStrippedBinary(), sd.getStyle("normal"));
			} catch (BadLocationException e) {
				mmodel.getLog().addMsg("Error: BadLocationException Caught When Trying To Clear Stripped Binary Text");
				return;
			}
			StyledDocument sdraw = rawbinary.getStyledDocument();
			try {
				sdraw.remove(0, sdraw.getLength());
				sdraw.insertString(sdraw.getLength(), s.getRawBinary(), sd.getStyle("normal"));
			} catch (BadLocationException e) {
				mmodel.getLog().addMsg("Error: BadLocationException Caught When Trying To Clear Raw Binary Text");
				return;
			}
		}
	}
	
	/**
	 * Updates the swipe console displaying messages
	 * @param s the swipe
	 */
	public void updateSwipeConsole(Swipe s){
		String[] ss = s.getErrorLog().getEntries(consoleLogCombo.getSelectedIndex());
		String[] formating = new String[ss.length];
		StyledDocument sd = console.getStyledDocument();
		addStyles(sd);

		for(int i=0;i<ss.length;i++){
			String curr = ss[i];
			if(curr.startsWith("Debug:")){ formating[i] = "debug"; } 
			if(curr.startsWith("Info:")){ formating[i] = "info"; }
			if(curr.startsWith("Warning:")){ formating[i] = "warning"; }
			if(curr.startsWith("Error:")){ formating[i] = "error"; }
			try{
				sd.insertString(sd.getLength(), ss[i], sd.getStyle(formating[i]));
			}catch (BadLocationException e){
				mmodel.getLog().addMsg("Error: BadLocationException Caught When Trying To Update The Swipe's Log");
				return;
			}
		}	
	}
	
	/**
	 * Update the character list based on which swipe is selected
	 * @param s current swipe
	 */
	public void updateCharList(Swipe s){
		String stripped = s.getStrippedBinary();
		int bpc = s.getBitsPerChar();
		DefaultListModel newmodel = new DefaultListModel();//create a new model and fill it, working with the live copy can cause an exception
		//charListModel.removeAllElements();
		if(bpc > 0){
			for(int i=0;i<=stripped.length()-bpc;i+=bpc){
				newmodel.addElement(" "+stripped.substring(i,i+bpc-1)+" "+stripped.substring(i+bpc-1,i+bpc)+"      "+
						(bpc == 5 ? Swipe.decodeABA(stripped.substring(i,i+bpc), bpc, null) : Swipe.decodeIATA(stripped.substring(i,i+bpc), bpc, null)));
			}
		}
		charList.setModel(newmodel);
	}
	
	/**
	 * Update the card table
	 */
	public void updateCardTable(){
		cardTable.setModel(mmodel.getCardTableModel());
		cardTable.setOpaque(true);cardTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		cardTable.getColumnModel().getColumn(CardTableModel.ID).setPreferredWidth(20);
		cardTable.getColumnModel().getColumn(CardTableModel.ENABLED).setPreferredWidth(40);
		cardTable.getColumnModel().getColumn(CardTableModel.HASHEDDATA).setPreferredWidth(380);
		cardTable.getColumnModel().getColumn(CardTableModel.DIRECTION).setPreferredWidth(80);
		cardTable.getColumnModel().getColumn(CardTableModel.STARTTIME).setPreferredWidth(80);
		cardTable.getColumnModel().getColumn(CardTableModel.ENDTIME).setPreferredWidth(80);
		cardTable.getColumnModel().getColumn(CardTableModel.USES).setPreferredWidth(30);
		String[] opts = {"Both", "Forward", "Backward"};
		cardTable.getColumnModel().getColumn(CardTableModel.DIRECTION).setCellEditor(new DefaultCellEditor(new JComboBox(opts)));
	}
	
	/**
	 * Removes the current model for the table so, defaultcellremover isnt present so we can serialze the data
	 */
	public void clearTableEditing(){
		table.setModel(new UserTableModel(mmodel));
	}
	
	/**
	 * After we serialzed the table, reset the table model 
	 */
	public void unclearTableEditing(){
		table.setModel(mmodel.getTableModel());
		table.setOpaque(true);table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getColumnModel().getColumn(UserTableModel.ID).setPreferredWidth(20);
		table.getColumnModel().getColumn(UserTableModel.ENABLED).setPreferredWidth(40);
		table.getColumnModel().getColumn(UserTableModel.FIRSTNAME).setPreferredWidth(200);
		table.getColumnModel().getColumn(UserTableModel.LASTNAME).setPreferredWidth(200);
		table.getColumnModel().getColumn(UserTableModel.NUMCARDS).setPreferredWidth(70);
		table.getColumnModel().getColumn(UserTableModel.UNLOCKTIME).setPreferredWidth(80);
		table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
		table.repaint();
	}
	
	/**
	 * Update the system console's log
	 */
	public synchronized void updateSystemConsole(){
		sd = systemText.getStyledDocument();
		synchronized(sd){//if many messages are added together it can cause problems
			String[] ss = mmodel.getLog().getEntries(systemLogCombo.getSelectedIndex());
			String[] formating = new String[ss.length];
			addStyles(sd);
			for(int i=0;i<ss.length;i++){
				String curr = ss[i];
				//System.out.print(i+" "+" "+ss.length+" "+formating.length+" "+curr);
				if(curr.startsWith("Debug:")){ formating[i] = "debug"; }
				if(curr.startsWith("Info:")){ formating[i] = "info"; }
				if(curr.startsWith("Warning:")){ formating[i] = "warning"; }
				if(curr.startsWith("Error:")){ formating[i] = "error"; }
				try{
					sd.insertString(sd.getLength(), ss[i], sd.getStyle(formating[i]));
				}catch (BadLocationException e){
					mmodel.getLog().addMsg("Error: BadLocationException Caught When Trying To Update The Swipe's Log");
					return;
				}
			}	

			//System.out.println(systemText.getText().length()+" "+strings.length+" "+(systemText.getText().length()-strings.length));
			systemText.setCaretPosition(sd.getLength());
		}
	}
	
	/**
	 * get the current swipe selected's timestamp
	 * @return the timestamp of the current swipe
	 */
	public String getSwipeSelected(){
		return (swipeList.getSelectedValue() == null ? null : swipeList.getSelectedValue().toString());
	}
	
	/**
	 * Return the oldest swipe stored, used to remove it when we hit the maximum swipes to store
	 * @return the oldest swipe's timestamp
	 */
	public String getOldestSwipe(){
		return (String)swipeListModel.lastElement();
	}
	
	/**
	 * Remove the oldest swipe in the stored list
	 */
	public void removeOldestSwipe(){
		swipeListModel.remove(swipeListModel.size()-1);
	}
	
	/**
	 * Return the row selected in the user table
	 * @return the current row selected
	 */
	public int getUserTableRowSelected(){
		return (table.getSelectedRowCount() == 1 ? table.getSelectedRow() : -1);
	}
	
	/**
	 * Returns the row selected in the card table
	 * @return selected row
	 */
	public int getCardTableRowSelected(){
		return cardTable.getSelectedRowCount() == 1 ? cardTable.getSelectedRow() : -1;
	}
	
	/**
	 * sets the number of users in the table for the status bar
	 * @param i the number of users
	 */
	public void updateUserCount(){
		userCount.setText(" "+mmodel.getTableModel().getRowsDefined()+" Users In Table  ");
	}
	
	/**
	 * Sets the number of swipes open
	 * @param i new value
	 */
	public void setSwipeCount(int i){
		swipeCount.setText(" "+i+" Swipes Recorded  ");
	}
	
	/**
	 * sets the number of cards in the table for the status bar
	 * @param i the number of cards
	 */
	public void updateCardCount(){
		cardCount.setText(" "+mmodel.getCardTableModel().getTotalCards()+" Cards In Table  ");
	}
	
	/**
	 * Adds styles for all the textpanes. Includes different styles like gray backgrounds, bold, grey text, teal text, and all of the error message colors
	 * @param doc
	 */
	private void addStyles(StyledDocument doc){
		try {
			doc.remove(0, doc.getLength());
		} catch (BadLocationException e) {
			mmodel.getLog().addMsg("Error: Error Removing Old Styled Document For 'Binary' Text Box");
			return;
		}
		Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
		Style normal = doc.addStyle("normal", def);
		doc.addStyle("normal1", def);
		StyleConstants.setFontFamily(def, "Monospaced");
		
		Style lightnormal = doc.addStyle("lightnormal", normal);//crc
		StyleConstants.setForeground(lightnormal, new Color(51,204,255));
		
		Style greyback = doc.addStyle("greyback", normal);//grey background
		StyleConstants.setBackground(greyback, new Color(221,221,221));
		
		Style boldnormal = doc.addStyle("boldnormal", normal); //selected
		StyleConstants.setBold(boldnormal, true);
		Style boldnormal1 = doc.addStyle("boldnormal1", normal); //selected
		StyleConstants.setBold(boldnormal1, true);
		
		Style boldlightnormal = doc.addStyle("boldlightnormal", normal); //selected crc
		StyleConstants.setBold(boldlightnormal, true);
		StyleConstants.setForeground(boldlightnormal, new Color(51,204,255));
		
		Style boldgreyback = doc.addStyle("boldgreyback", normal); //selected grey background
		StyleConstants.setBold(boldgreyback, true);
		StyleConstants.setBackground(boldgreyback, new Color(221,221,221));
		
		Style boldlightgreyback = doc.addStyle("boldlightgreyback", normal); //selected crc grey background
		StyleConstants.setBold(boldlightgreyback, true);
		StyleConstants.setBackground(boldlightgreyback, new Color(221,221,221));
		StyleConstants.setForeground(boldlightgreyback, new Color(51,204,255));
		
		Style lightgreyback = doc.addStyle("lightgreyback", normal); //crc grey background
		StyleConstants.setBackground(lightgreyback, new Color(221,221,221));
		StyleConstants.setForeground(lightgreyback, new Color(51,204,255));
		//start console fonts
		Style debug = doc.addStyle("debug", normal);
		StyleConstants.setForeground(debug, new Color(0,187,0));
		Style info = doc.addStyle("info", normal);
		StyleConstants.setForeground(info, new Color(0,153,255));
		Style warning = doc.addStyle("warning", normal);
		StyleConstants.setForeground(warning, new Color(255,119,0));
		Style error = doc.addStyle("error", normal);
		StyleConstants.setForeground(error, new Color(221,0,0));
	}
	
	/**
	 * Sets a tab as selected
	 * @param i the tab to select
	 */
	public void setTabSelected(int i){
		tabs.setSelectedIndex(i);
	}
	
	/**
	 * Sets a row as selected in the user table
	 * @param i the row
	 */
	public void setTableSelection(int i){
		table.getSelectionModel().setSelectionInterval(i, i);
	}
	
	public void lockScreen(boolean b){
		tabs.setVisible(!b);
	}
	
	//TODO
	public void setTabNewData(int i){
		tabs.setBackgroundAt(i, new Color(0xDDFFDD));
	}
}