/*
This file is part of Magstripper, Copyright 2006, 2007.

Magstripper is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Magstripper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package magstripper;
//import java.util.Enumeration;
import gnu.io.CommPortIdentifier;
import gnu.io.NoSuchPortException;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;

public class Strike extends Thread{
	private int opentime;
	private String comport;
	private static CommPortIdentifier portIdent;
	private volatile SerialPort sp;
	private static volatile int unlocktime = 0;
	private MagstripperModel model;
	
	/**
	 * Overloaded class constructor specifying the unlock time and model
	 * @param m the current model used
	 * @param time the unlock time
	 */
	public Strike(MagstripperModel m, int time){
		this(m);
		opentime = time;
	}
	
	/**
	 * Class constructor specifying the model
	 * @param m the current model
	 */
	public Strike(MagstripperModel m){
		model = m;
		opentime = m.getLockTime();
		comport = m.getComPort();
		portIdent = null;
		try{
			portIdent = CommPortIdentifier.getPortIdentifier(comport);
		}catch(NoSuchPortException nspe){
			model.getLog().addMsg("Error: Could Not Access Port "+comport+", NoSuchPortException Caught");
			return;
		}catch(UnsatisfiedLinkError ule){
			model.getLog().addMsg("Error: RXTXcomm Is Not Installed Properly For Serial Port Access");
			return;
		}
	}
	
	/**
	 * The method that actually unlocks the lock. Called by run()
	 */
	public void unlock(){
		if(portIdent == null){ //make sure the port was defined
			model.getLog().addMsg("Error: Can Not Unlock The Strike On Port "+comport);
			unlocktime = 0;
			return;
		}
		synchronized(portIdent){ //only let one thread unlock at a time
			model.setLockState(false); //tell the model the lock is unlocked
			try {
				sp = (SerialPort)portIdent.open(comport,300); //unlock the lock
			}catch(PortInUseException piue) {model.getLog().addMsg("Error: Can't Unlock, Port Is In Use");return;}
			int temptime = unlocktime;
			while(temptime > 0){ //while the lock needs to be open
				try {
					Thread.sleep(temptime); //sleep for as long as the lock should be open
				}catch(InterruptedException ie) {model.getLog().addMsg("Error: InturruptedException Caught During Strike Open Period");return;}
				unlocktime-=temptime; //decrement the unlock time by how long we unlocked it for
				temptime = unlocktime; //set temptime to how long we should unlock (incase another thread spawned while the lock was opened)
			}
			sp.close(); //must relock it temporairaly due to PortIdent resources not being shared between threads
			model.setLockState(true);
		}
	}
	
	/**
	 * This method is executed when a new threaded is started.
	 */
	public void run(){
		unlocktime+=opentime; //increment the unlock time by how long we should keep it open
		if(unlocktime <= opentime)//unlock the door since it was locked before we incremented it
			this.unlock();
	}
	
	/**
	 * Used to check if the lock is already being used
	 * @return true if it is in use, false otherwise
	 */
	public boolean inUse(){
		return portIdent.isCurrentlyOwned();
	}
	
//	/**
//	 * List the ports available to systemout
//	 */
//	private static void listPorts(){
//        Enumeration portEnum = CommPortIdentifier.getPortIdentifiers();
//        while ( portEnum.hasMoreElements() ) 
//        {
//            CommPortIdentifier portIdentifier = (CommPortIdentifier) portEnum.nextElement();
//            System.out.println(portIdentifier.getName() + " - " + portIdentifier.getPortType());
//        }        
//    }
	
	
}